@@ -24,6 +24,8 @@
 
 /**
  * @file ports.h
+ * Collection of functions for ports.
+ * This includes dispatchin, reading metadata etc.
  */
 
 #ifndef RTOSC_PORTS
@@ -54,8 +56,7 @@ struct RtData
     RtData(void);
 
     /**
-     * @brief location of where the dispatch routine is currently being called
-     *
+     * Location of where the dispatch routine is currently being called.
      * If non-NULL, the dispatch routine will update the port name here while
      * walking through the Ports tree
      */
@@ -181,14 +182,20 @@ struct Ports
     Ports(const Ports&) = delete;
 
     /**
-     * Dispatches message to all matching ports.
-     * This uses simple pattern matching available in rtosc::match.
+     * @brief Dispatches message to all matching ports.
+     *
+     * This uses simple pattern matching available in rtosc::match().
+     *
+     * @invariant In each recursion, "d.loc + m" must make the port's full path.
+     *   For further recursions, a call to scat() will append
+     *   the toplevel of m to d.loc, and in the rRecur* callbacks, SNIP() will
+     *   cut the toplevel from m. I.e.: loc="", m="a/b/c" => loc="/a/", m="b/c".
      *
      * @param m A valid OSC message. Note that the address part shall not
      *          contain any type specifier.
-     * @param d The RtData object shall contain a path buffer (or null), the length of
-     *          the buffer, a pointer to data. It must also contain the location
-     *          if an answer is being expected.
+     * @param d The RtData object shall contain a path buffer (or null), the
+     *          length of the buffer, a pointer to data. It must also contain
+     *          the location if an answer is being expected.
      * @param base_dispatch Whether the OSC path is to be interpreted as a full
      *                      OSC path beginning at the root
      */
@@ -202,9 +209,9 @@ struct Ports
 
 
     /**
-     * Find the best match for a given path
+     * @brief Find the best match for a given path
      *
-     * @parameter path partial OSC path
+     * @param path partial OSC path
      * @returns first path prefixed by the argument
      *
      * Example usage:
@@ -253,7 +260,7 @@ struct MergePorts:public Ports
 };
 
 /**
- * @brief Returns a port's default value
+ * Return a port's default value
  *
  * Returns the default value of a given port, if any exists, as a string.
  * For the parameters, see the overloaded function.
@@ -266,7 +273,7 @@ const char* get_default_value(const char* port_name, const Ports& ports,
                               int32_t idx = -1, int recursive = 1);
 
 /**
- * @brief Returns a port's default value
+ * Return a port's default value
  *
  * Returns the default value of a given port, if any exists, as an array of
  * rtosc_arg_vals . The values in the resulting array are being canonicalized,
@@ -299,19 +306,22 @@ int get_default_value(const char* port_name, const char *port_args,
 
 
 /**
- * @brief Return a string list of all changed values
+ * Return a string list of all changed values
  *
  * Return a human readable list of the value that changed
  * corresponding to the rDefault macro
  * @param ports The static ports structure
  * @param runtime The runtime object
+ * @note This function is not realtime save (It uses std::string), which is
+ *   usually OK, since this function is being run inside a non-RT thread. If you
+ *   need this to be realtime save, add a template parameter for a functor that
+ *   replaces the std::string handling.
  * @return The list of ports and their changed values, linewise
  */
 std::string get_changed_values(const Ports& ports, void* runtime);
 
-//! @brief Class to modify messages loaded from savefiles
-//!
-//! Object of this class shall be passed to savefile loading routines. You can
+//! @brief Class to modify and dispatch messages loaded from savefiles.
+//! Objects of this class shall be passed to savefile loading routines. You can
 //! inherit to change the behaviour, e.g. to modify or discard such messages.
 class savefile_dispatcher_t
 {
@@ -337,7 +347,7 @@ protected:
                   app_curver; //!< current app version
 
     //! call this to dispatch a message
-    void operator()(const char* msg);
+    bool operator()(const char* msg) { return do_dispatch(msg); }
 
     static int default_response(size_t nargs, bool first_round,
                                 dependency_t dependency);
@@ -349,6 +359,8 @@ private:
                             size_t maxargs, size_t nargs,
                             rtosc_arg_val_t* args,
                             bool round2, dependency_t dependency);
+    //! call this to dispatch a message
+    virtual bool do_dispatch(const char* msg);
 
     friend int dispatch_printed_messages(const char* messages,
                                          const Ports& ports, void* runtime,
@@ -362,8 +374,7 @@ private:
 };
 
 /**
- * @brief Scan OSC messages from human readable format and dispatch them
- *
+ * Scan OSC messages from human readable format and dispatch them.
  * @param messages The OSC messages, whitespace-separated
  * @param ports The static ports structure
  * @param runtime The runtime object
@@ -371,6 +382,7 @@ private:
  *   You can overwrite its virtual functions, and you should specify any of the
  *   version structs if needed. All other members shall not be initialized.
  * @return The number of messages read, or, if there was a read error,
+ *   or the dispatcher did refuse to dispatch,
  *   the number of bytes read until the read error occured minus one
  */
 int dispatch_printed_messages(const char* messages,
@@ -378,9 +390,7 @@ int dispatch_printed_messages(const char* messages,
                               savefile_dispatcher_t *dispatcher = NULL);
 
 /**
- * @brief Return a savefile containing all values that differ from the default
- *   values.
- *
+ * Return a savefile containing all values that differ from the default values.
  * @param ports The static ports structure
  * @param runtime The runtime object
  * @param appname Name of the application calling this function
@@ -391,8 +401,7 @@ std::string save_to_file(const Ports& ports, void* runtime,
                          const char* appname, rtosc_version appver);
 
 /**
- * @brief Read save file and dispatch contained parameters
- *
+ * Read save file and dispatch contained parameters.
  * @param file_content The file as a C string
  * @param ports The static ports structure
  * @param runtime The runtime object
@@ -410,9 +419,9 @@ int load_from_file(const char* file_content,
                    savefile_dispatcher_t* dispatcher = NULL);
 
 /**
- * @brief Convert given argument values to their canonical representation, e.g.
- * to the ports first (or-wise) argument types.
+ * Convert given argument values to their canonical representation.
  *
+ * The ports first (or-wise) argument types are defined as canonical.
  * E.g. if passing two 'S' argument values, the
  * port could be `portname::ii:cc:SS` or `portname::ii:t`.
  *
@@ -428,8 +437,7 @@ int canonicalize_arg_vals(rtosc_arg_val_t* av, size_t n,
                           const char* port_args, Port::MetaContainer meta);
 
 /**
- * @brief Converts each of the given arguments to their mapped symbol, if
- *        possible
+ * Convert each of the given arguments to their mapped symbol, if possible.
  * @param av The input and output argument values
  * @param n The size of @p av
  * @param meta The port's metadata container
@@ -442,7 +450,7 @@ void map_arg_vals(rtosc_arg_val_t* av, size_t n,
  *********************/
 //typedef std::function<void(const Port*,const char*)> port_walker_t;
 /**
- * @brief function pointer type for port walking
+ * Function pointer type for port walking.
  *
  * accepts:
  * - the currently walked port
@@ -458,23 +466,29 @@ typedef void(*port_walker_t)(const Port*,const char*,const char*,
                              const Ports&,void*,void*);
 
 /**
- * @brief Call a function on all ports and subports
+ * Call a function on all ports and subports.
  * @param base The base port of traversing
  * @param name_buffer Buffer which will be filled with the port name; must be
  *   reset to zero over the full length!
  * @param buffer_size Size of name_buffer
  * @param data Data that should be available in the callback
  * @param walker Callback function
+ * @param expand_bundles Whether walking over bundles without subports
+ *   invokes walking over each of the bundle's port
+ * @param runtime Runtime object corresponding to @param base. If given, checks
+ *   the runtime object will be used stop recursion if the Ports are disabled
+ *   (using the "enabled by" property) and it will be passed to the walker
  */
 void walk_ports(const Ports *base,
                 char          *name_buffer,
                 size_t         buffer_size,
                 void          *data,
                 port_walker_t  walker,
+                bool expand_bundles = true,
                 void *runtime = NULL);
 
 /**
- * @brief Return the index with value @p value from the metadata's enumeration
+ * Return the index with value @p value from the metadata's enumeration.
  * @param meta The metadata
  * @param value The value to search the key for
  * @return The first key holding value, or `std::numeric_limits<int>::min()`
