@@ -1,4 +1,4 @@
-#include "../../include/rtosc/ports.h"
+﻿#include "../../include/rtosc/ports.h"
 #include "../../include/rtosc/rtosc.h"
 #include "../../include/rtosc/pretty-format.h"
 
@@ -42,13 +42,13 @@ static inline void scat(char *dest, const char *src)
 RtData::RtData(void)
     :loc(NULL), loc_size(0), obj(NULL), matches(0), message(NULL)
 {
-    for(int i=0; i<sizeof(idx)/sizeof(int); ++i)
+    for(size_t i=0; i<sizeof(idx)/sizeof(int); ++i)
         idx[i] = 0;
 }
 
 void RtData::push_index(int ind)
 {
-    for(int i=1; i<sizeof(idx)/sizeof(int); ++i)
+    for(size_t i=1; i<sizeof(idx)/sizeof(int); ++i)
         idx[i] = idx[i-1];
     idx[0] = ind;
 }
@@ -675,11 +675,8 @@ class CapturePretty : public RtData
         assert(wrt);
     }*/
 
-    void reply(const char *, const char *args, ...) override
-    {        
-        va_list va;
-        va_start(va,args);
-
+    void reply_va(const char *args, va_list va)
+    {
         size_t nargs = strlen(args);
         rtosc_arg_val_t arg_vals[nargs];
 
@@ -688,10 +685,25 @@ class CapturePretty : public RtData
         size_t wrt = rtosc_print_arg_vals(arg_vals, nargs,
                                           buffer, buffersize, NULL,
                                           cols_used);
-        va_end(va);
         assert(wrt);
     }
 
+    void broadcast(const char *, const char *args, ...) override
+    {
+        va_list va;
+        va_start(va,args);
+        reply_va(args, va);
+        va_end(va);
+    }
+
+    void reply(const char *, const char *args, ...) override
+    {        
+        va_list va;
+        va_start(va,args);
+        reply_va(args, va);
+        va_end(va);
+    }
+
 public:
     //! Return the argument values, pretty-printed
     const char* value() const { return buffer; }
@@ -771,16 +783,27 @@ class Capture : public RtData
         nargs = cur_idx;
     }
 
-    void reply(const char *, const char *args, ...) override
+    void reply_va(const char *args, va_list va)
     {
-        va_list va;
-        va_start(va,args);
-
         nargs = strlen(args);
         assert((size_t)nargs <= max_args);
 
         rtosc_v2argvals(arg_vals, nargs, args, va);
+    }
 
+    void broadcast(const char *, const char *args, ...) override
+    {
+        va_list va;
+        va_start(va, args);
+        reply_va(args, va);
+        va_end(va);
+    }
+
+    void reply(const char *, const char *args, ...) override
+    {
+        va_list va;
+        va_start(va,args);
+        reply_va(args, va);
         va_end(va);
     }
 public:
@@ -939,34 +962,65 @@ int rtosc::canonicalize_arg_vals(rtosc_arg_val_t* av, size_t n,
                                  const char* port_args,
                                  Port::MetaContainer meta)
 {
-    const char* first = port_args;
+    const char* first0 = port_args;
     int errors_found = 0;
 
-    for( ; *first && (*first == ':' || *first == '[' || *first == ']');
-           ++first) ;
+    // skip "[]:"
+    for( ; *first0 && (*first0 == ':' || *first0 == '[' || *first0 == ']');
+           ++first0) ;
 
-    for(size_t i = 0; i < n; ++i, ++first, ++av)
+    size_t arr_size;
+    size_t max;
+    bool is_array;
+    rtosc_arg_val_t* start = av;
+    if(av->type == 'a') {
+        arr_size = av->val.a.len;
+        ++av;
+        max = 1; // only one element per bundle element
+                 // TODO: multiple may be possible
+        is_array = true;
+    }
+    else {
+        arr_size = 1;
+        max = n;
+        is_array = false;
+    }
+
+    for(size_t a = 0; a < arr_size; ++a)
     {
-        for( ; *first && (*first == '[' || *first == ']'); ++first) ;
-
-        if(!*first || *first == ':')
+        const char* first = first0;
+        for(size_t i = 0; i < max; ++i, ++first, ++av)
         {
-            // (n-i) arguments left, but we have no recipe to convert them
-            return n-i;
-        }
+            // skip "[]"
+            for( ; *first && (*first == '[' || *first == ']'); ++first) ;
 
-        if(av->type == 'S' && *first == 'i')
-        {
-            int val = enum_key(meta, av->val.s);
-            if(val == std::numeric_limits<int>::min())
-                ++errors_found;
-            else
+            assert(!strchr(first0, '#'));
+
+//            if(is_array) // TODO: currently, only one element per bundle element
+//                assert(first[1] == 0);
+
+            if(!*first || *first == ':')
             {
-                av->type = 'i';
-                av->val.i = val;
+                // (n-i) arguments left, but we have no recipe to convert them
+                return n-i;
+            }
+
+            if(av->type == 'S' && *first == 'i')
+            {
+                int val = enum_key(meta, av->val.s);
+                if(val == std::numeric_limits<int>::min())
+                    ++errors_found;
+                else
+                {
+                    av->type = 'i';
+                    av->val.i = val;
+                }
             }
         }
     }
+    if(is_array && arr_size)
+        start->val.a.type = av[-1].type;
+
     return errors_found;
 }
 
@@ -1010,12 +1064,14 @@ int rtosc::get_default_value(const char* port_name, const char* port_args,
         rtosc_scan_arg_vals(pretty, res, nargs, strbuf, strbufsize);
 
         {
+            // TODO: port_hint could be NULL here!
             int errs_found = canonicalize_arg_vals(res,
                                                    nargs,
                                                    port_args,
                                                    port_hint->meta());
             if(errs_found)
-                fprintf(stderr, "Could not canonicalize %s\n", pretty);
+                fprintf(stderr, "Could not canonicalize %s for port %s\n",
+                        pretty, port_name);
             assert(!errs_found); // error in the metadata?
         }
     }
@@ -1025,6 +1081,58 @@ int rtosc::get_default_value(const char* port_name, const char* port_args,
     return nargs;
 }
 
+// TODO: pass base as pointer, so it can be null?
+template<class F>
+void bundle_foreach(const Port& p, char* old_end,
+                    /* data which is only being used by the ftors */
+                    const char* name_buffer, const Ports& base,
+                    void* data, void* runtime, const F& ftor,
+                    /* options */
+                    bool expand_bundles = true,
+                    bool cut_afterwards = true)
+{
+    const char *name = p.name;
+    char       *pos  = old_end;
+    while(*name != '#') *pos++ = *name++;
+    const unsigned max = atoi(name+1);
+    while(isdigit(*++name)) ;
+
+    char* pos2;
+
+    if(expand_bundles)
+    for(unsigned i=0; i<max; ++i)
+    {
+        const char* name2_2 = name;
+        pos2 = pos + sprintf(pos,"%d",i);
+
+        // append everything behind the '#' (for cases like a#N/b)
+        while(*name2_2 && *name2_2 != ':')
+            *pos2++ = *name2_2++;
+
+        ftor(&p, name_buffer, old_end, base, data, runtime);
+    }
+    else
+    {
+        const char* name2_2 = name;
+        pos2 = pos;
+
+        // append everything behind the '#' (for cases like a#N/b)
+        while(*name2_2 && *name2_2 != ':')
+            *pos2++ = *name2_2++;
+
+        ftor(&p, name_buffer, old_end, base, data, runtime);
+    }
+
+    if(cut_afterwards)
+        *old_end = 0;
+    else
+        *pos2 = 0;
+}
+
+// use this function if you don't want to do anything in bundle_foreach
+void bundle_foreach_do_nothing(const Port*, const char*, const char*,
+                               const Ports&, void*, void*){}
+
 std::string rtosc::get_changed_values(const Ports& ports, void* runtime)
 {
     std::string res;
@@ -1041,6 +1149,13 @@ std::string rtosc::get_changed_values(const Ports& ports, void* runtime)
     {
         assert(runtime);
         const Port::MetaContainer meta = p->meta();
+#if 0
+// practical for debugging if a parameter was changed, but not saved
+        if(!strncmp(port_buffer, "/part0/partefx2/Phaser/", 23))
+        {
+            puts("break here");
+        }
+#endif
 
         if((p->name[strlen(p->name)-1] != ':' && !strstr(p->name, "::"))
             || meta.find("parameter") == meta.end())
@@ -1049,19 +1164,33 @@ std::string rtosc::get_changed_values(const Ports& ports, void* runtime)
             // thus, it can not be compared with the default value
             return;
         }
+        else
+        { // TODO: duplicate to above? (colon[1])
+            const char* colon = strchr(p->name, ':');
+            if(!colon || !colon[1])
+            {
+                // runtime information can not be loaded, so don't save it
+                // a possible FEATURE would be to save it anyways
+                return;
+            }
+        }
 
-        char loc[buffersize] = "";
+        char loc[buffersize] = ""; // buffer to hold the dispatched path
         rtosc_arg_val_t arg_vals_default[max_arg_vals];
         rtosc_arg_val_t arg_vals_runtime[max_arg_vals];
+        // buffer to hold the message (i.e. /port ..., without port's bases)
         char buffer_with_port[buffersize];
-        char cur_value_pretty[buffersize] = " ";
         char strbuf[buffersize]; // temporary string buffer for pretty-printing
 
         std::string* res = (std::string*)data;
         assert(strlen(port_buffer) + 1 < buffersize);
-        strncpy(loc, port_buffer, buffersize); // TODO: +-1?
+        // copy the path until before the message
+        strncpy(loc, port_buffer, std::min((ptrdiff_t)buffersize,
+                                           port_from_base - port_buffer));
+        char* loc_end = loc + (port_from_base - port_buffer);
+        size_t loc_remain_size = buffersize - (port_from_base - port_buffer);
+        *loc_end = 0;
 
-        strncpy(buffer_with_port, port_from_base, buffersize);
         const char* portargs = strchr(p->name, ':');
         if(!portargs)
             portargs = p->name + strlen(p->name);
@@ -1084,38 +1213,163 @@ std::string rtosc::get_changed_values(const Ports& ports, void* runtime)
                                               arg_vals_default,
                                               strbuf,
                                               buffersize);
-        size_t nargs_runtime = get_value_from_runtime(runtime,
-                                                      *p,
-                                                      buffersize, loc,
-                                                      port_from_base,
-                                                      buffer_with_port,
-                                                      buffersize,
-                                                      max_arg_vals,
-                                                      arg_vals_runtime);
 
-        if(nargs_default == nargs_runtime)
+        if(nargs_default > 0)
         {
+            size_t nargs_runtime = 0;
+
+            auto ftor = [&](const Port* p, const char* name_buffer,
+                            const char* old_end,
+                            const Ports& ,void* ,void* runtime)
+            {
+                // TODO: strncpy is slow, replace it everywhere
+                strncpy(buffer_with_port, p->name, buffersize);
+
+                // the caller of ftor (in some cases bundle_foreach) has
+                // already filled old_end correctly, but we have to copy this
+                // over to loc_end
+                strncpy(loc_end, old_end, loc_remain_size);
+
+                size_t
+                nargs_runtime_cur = get_value_from_runtime(runtime,
+                                                           *p,
+                                                           buffersize, loc,
+                                                           old_end,
+                                                           buffer_with_port,
+                                                           buffersize,
+                                                           max_arg_vals,
+                                                           arg_vals_runtime +
+                                                               nargs_runtime);
+                nargs_runtime += nargs_runtime_cur;
+            };
+
+            auto refix_old_end = [&base](const Port* _p, char* _old_end)
+            { // TODO: remove base capture
+                bundle_foreach(*_p, _old_end, NULL,
+                               base, NULL, NULL, bundle_foreach_do_nothing,
+                               false, false);
+            };
+
+            if(strchr(p->name, '#'))
+            {
+                // idea:
+                //                    p/a/b
+                // bundle_foreach =>  p/a#0/b, p/a#1/b, ... p/a#n/b, p
+                // bundle_foreach =>  p/a/b
+                // => justification for const_cast
+
+                // Skip the array element (type 'a') for now...
+                ++nargs_runtime;
+
+                // Start filling at arg_vals_runtime + 1
+                char* old_end_noconst = const_cast<char*>(port_from_base);
+                bundle_foreach(*p, old_end_noconst, port_buffer + 1,
+                               base, data, runtime,
+                               ftor, true);
+
+                // glue the old end behind old_end_noconst again
+                refix_old_end(p, old_end_noconst);
+
+                // "Go back" to fill arg_vals_runtime + 0
+                arg_vals_runtime[0].type = 'a';
+                arg_vals_runtime[0].val.a.len = nargs_runtime-1;
+                arg_vals_runtime[0].val.a.type = arg_vals_runtime[1].type;
+            }
+            else
+                ftor(p, port_buffer, port_from_base, base, NULL, runtime);
+
+#if 0
+// practical for debugging if a parameter was changed, but not saved
+            if(!strncmp(port_buffer, "/part0/partefx2/Phaser/", 23))
+            {
+                puts("break here");
+            }
+#endif
             canonicalize_arg_vals(arg_vals_default, nargs_default,
                                   strchr(p->name, ':'), meta);
-            if(!rtosc_arg_vals_eq(arg_vals_default,
-                                  arg_vals_runtime,
-                                  nargs_default,
-                                  nargs_runtime,
-                                  NULL))
-            {
-                map_arg_vals(arg_vals_runtime, nargs_runtime, meta);
-                rtosc_print_arg_vals(arg_vals_runtime, nargs_runtime,
-                                     cur_value_pretty + 1, buffersize - 1,
-                                     NULL, strlen(port_buffer) + 1);
 
-                *res += port_buffer;
-                *res += cur_value_pretty;
-                *res += "\n";
+            auto write_msg = [&res, &meta, &port_buffer]
+                                 (const rtosc_arg_val_t* arg_vals_default,
+                                  rtosc_arg_val_t* arg_vals_runtime,
+                                  int nargs_default, size_t nargs_runtime)
+            {
+                if(!rtosc_arg_vals_eq(arg_vals_default, arg_vals_runtime,
+                                      nargs_default, nargs_runtime, nullptr))
+                {
+                    char cur_value_pretty[buffersize] = " ";
+
+                    map_arg_vals(arg_vals_runtime, nargs_runtime, meta);
+
+                    rtosc_print_arg_vals(arg_vals_runtime, nargs_runtime,
+                                         cur_value_pretty + 1, buffersize - 1,
+                                         NULL, strlen(port_buffer) + 1);
+                    *res += port_buffer;
+                    *res += cur_value_pretty;
+                    *res += "\n";
+                }
+            }; // functor write_msg
+
+            if(arg_vals_runtime[0].type == 'a' && strchr(port_from_base, '/'))
+            {
+                // These are grouped as an array, but the port structure
+                // implicits that they shall be handled as single values
+                // inside their subtrees
+                //  => We don't print this as an array
+                //  => All arrays in savefiles have their numbers after
+                //     the last port separator ('/')
+
+                // used if the value of lhs or rhs is range-computed:
+                rtosc_arg_val_t rlhs, rrhs;
+
+                rtosc_arg_val_t_const_itr litr, ritr;
+                rtosc_arg_val_itr_init(&litr, arg_vals_default+1);
+                rtosc_arg_val_itr_init(&ritr, arg_vals_runtime+1);
+
+                auto write_msg_adaptor = [&litr, &ritr,&rlhs,&rrhs,&write_msg](
+                    const Port* p,
+                    const char* port_buffer, const char* old_end,
+                    const Ports&, void*, void*)
+                {
+                    const rtosc_arg_val_t
+                        * lcur = rtosc_arg_val_itr_get(&litr, &rlhs),
+                        * rcur = rtosc_arg_val_itr_get(&ritr, &rrhs);
+
+                    if(!rtosc_arg_vals_eq_single(
+                            rtosc_arg_val_itr_get(&litr, &rlhs),
+                            rtosc_arg_val_itr_get(&ritr, &rrhs), nullptr))
+                    {
+                        auto get_sz = [](const rtosc_arg_val_t* a) {
+                            return a->type == 'a' ? (a->val.a.len + 1) : 1; };
+                        // the const-ness does not matter
+                        write_msg(lcur,
+                            const_cast<rtosc_arg_val_t*>(rcur),
+                            get_sz(lcur), get_sz(rcur));
+                    }
+
+                    rtosc_arg_val_itr_next(&litr);
+                    rtosc_arg_val_itr_next(&ritr);
+                };
+
+                char* old_end_noconst = const_cast<char*>(port_from_base);
+
+                // iterate over the whole array
+                bundle_foreach(*p, old_end_noconst, port_buffer,
+                               base, NULL, NULL,
+                               write_msg_adaptor, true);
+
+                // glue the old end behind old_end_noconst again
+                refix_old_end(p, old_end_noconst);
+
+            }
+            else
+            {
+                write_msg(arg_vals_default, arg_vals_runtime,
+                          nargs_default, nargs_runtime);
             }
         }
     };
 
-    walk_ports(&ports, port_buffer, buffersize, &res, on_reach_port,
+    walk_ports(&ports, port_buffer, buffersize, &res, on_reach_port, false,
                runtime);
 
     if(res.length()) // remove trailing newline
@@ -1123,7 +1377,7 @@ std::string rtosc::get_changed_values(const Ports& ports, void* runtime)
     return res;
 }
 
-void rtosc::savefile_dispatcher_t::operator()(const char* msg)
+bool rtosc::savefile_dispatcher_t::do_dispatch(const char* msg)
 {
     *loc = 0;
     RtData d;
@@ -1131,12 +1385,13 @@ void rtosc::savefile_dispatcher_t::operator()(const char* msg)
     d.loc = loc; // we're always dispatching at the base
     d.loc_size = 1024;
     ports->dispatch(msg, d, true);
+    return !!d.matches;
 }
 
 int savefile_dispatcher_t::default_response(size_t nargs,
                                             bool first_round,
                                             savefile_dispatcher_t::dependency_t
-                                            dependency)
+                                                dependency)
 {
     // default implementation:
     // no dependencies  => round 0,
@@ -1166,6 +1421,7 @@ int rtosc::dispatch_printed_messages(const char* messages,
     int rd, rd_total = 0;
     int nargs;
     int msgs_read = 0;
+    bool ok = true;
 
     savefile_dispatcher_t dummy_dispatcher;
     if(!dispatcher)
@@ -1177,19 +1433,19 @@ int rtosc::dispatch_printed_messages(const char* messages,
     //  * in the second round, only dispatch those with ports that depend on
     //    other ports
     //  * in the first round, only dispatch all others
-    for(int round = 0; round < 2 && msgs_read >= 0; ++round)
+    for(int round = 0; round < 2 && ok; ++round)
     {
         msgs_read = 0;
         rd_total = 0;
         const char* msg_ptr = messages;
-        while(*msg_ptr && (msgs_read >= 0))
+        while(*msg_ptr && ok)
         {
             nargs = rtosc_count_printed_arg_vals_of_msg(msg_ptr);
             if(nargs >= 0)
             {
-                // 16 is usually too much, but it allows the user to add
-                // arguments if necessary
-                size_t maxargs = 16;
+                // nargs << 1 is usually too much, but it allows the user to use
+                // these values (using on_dispatch())
+                size_t maxargs = std::max(nargs << 1, 16);
                 rtosc_arg_val_t arg_vals[maxargs];
                 rd = rtosc_scan_message(msg_ptr, portname, buffersize,
                                         arg_vals, nargs, strbuf, buffersize);
@@ -1210,23 +1466,47 @@ int rtosc::dispatch_printed_messages(const char* messages,
                                                 round, dependency);
 
                 if(nargs == savefile_dispatcher_t::abort)
-                    msgs_read = -rd_total-1; // => causes abort
+                    ok = false;
                 else
                 {
                     if(nargs != savefile_dispatcher_t::discard)
                     {
+                        size_t max;
+                        const rtosc_arg_val_t* arg_val_ptr;
+                        bool is_array;
+                        if(nargs && arg_vals[0].type == 'a')
+                        {
+                            is_array = true;
+                            max = arg_vals[0].val.a.len;
+                            nargs = 1; // overwrite nargs
+                            arg_val_ptr = arg_vals + 1;
+                        }
+                        else {
+                            is_array = false;
+                            max = 1;
+                            arg_val_ptr = arg_vals;
+                        }
+
                         rtosc_arg_t vals[nargs];
                         char argstr[nargs+1];
-                        for(int i = 0; i < nargs; ++i) {
-                            vals[i] = arg_vals[i].val;
-                            argstr[i] = arg_vals[i].type;
+                        char* portname_end = portname + strlen(portname);
+
+                        for(size_t arr_idx = 0; arr_idx < max && ok; ++arr_idx)
+                        {
+                            for(int i = 0; i < nargs; ++i) {
+                                vals[i] = arg_val_ptr[arr_idx + i].val;
+                                argstr[i] = arg_val_ptr[arr_idx + i].type;
+                            }
+                            argstr[nargs] = 0;
+
+                            if(is_array)
+                                snprintf(portname_end, 8, "%d", (int)arr_idx);
+
+                            rtosc_amessage(message, buffersize, portname,
+                                           argstr, vals);
+
+                            ok = (*dispatcher)(message);
                         }
-                        argstr[nargs] = 0;
-
-                        rtosc_amessage(message, buffersize, portname,
-                                       argstr, vals);
-
-                        (*dispatcher)(message);
                     }
                 }
 
@@ -1240,13 +1520,11 @@ int rtosc::dispatch_printed_messages(const char* messages,
                 while(*++msg_ptr) ;
             }
             else {
-                // overwrite meaning of msgs_read in order to
-                // inform the user where the read error occurred
-                msgs_read = -rd_total-1;
+                ok = false;
             }
         }
     }
-    return msgs_read;
+    return ok ? msgs_read : -rd_total-1;
 }
 
 std::string rtosc::save_to_file(const Ports &ports, void *runtime,
@@ -1491,8 +1769,9 @@ MergePorts::MergePorts(std::initializer_list<const rtosc::Ports*> c)
  * @param loc The absolute path of @p port
  * @param loc_size The maximum usable size of @p loc
  * @param ports The Ports object containing @p port
- * @param runtime TODO
- * @return TODO
+ * @param runtime The runtime object (optional)
+ * @return True if no runtime is provided or @p port has no enabled property.
+ *         Otherwise, the state of the "enabled by" toggle
  */
 bool port_is_enabled(const Port* port, char* loc, size_t loc_size,
                      const Ports& base, void *runtime)
@@ -1535,7 +1814,6 @@ bool port_is_enabled(const Port* port, char* loc, size_t loc_size,
             char* collapsed_loc = Ports::collapsePath(loc);
             loc_size -= (collapsed_loc - loc);
 
-// TODO: collapse, use .. only in one case
             /*
                 receive the "enabled" property
              */
@@ -1569,12 +1847,14 @@ void rtosc::walk_ports(const Ports  *base,
                        size_t        buffer_size,
                        void         *data,
                        port_walker_t walker,
+                       bool          expand_bundles,
                        void*         runtime)
 {
     auto walk_ports_recurse = [](const Port& p, char* name_buffer,
                                  size_t buffer_size, const Ports& base,
                                  void* data, port_walker_t walker,
-                                 void* runtime, const char* old_end)
+                                 void* runtime, const char* old_end,
+                                 bool expand_bundles)
     {
         // TODO: all/most of these checks must also be done for the
         // first, non-recursive call
@@ -1610,7 +1890,7 @@ void rtosc::walk_ports(const Ports  *base,
         }
         if(enabled)
             rtosc::walk_ports(p.ports, name_buffer, buffer_size,
-                              data, walker, runtime);
+                              data, walker, expand_bundles, runtime);
     };
 
     //only walk valid ports
@@ -1646,7 +1926,8 @@ void rtosc::walk_ports(const Ports  *base,
 
                     //Recurse
                     walk_ports_recurse(p, name_buffer, buffer_size,
-                                       *base, data, walker, runtime, old_end);
+                                       *base, data, walker, runtime, old_end,
+                                       expand_bundles);
                 }
             } else {
                 //Append the path
@@ -1655,28 +1936,13 @@ void rtosc::walk_ports(const Ports  *base,
 
                 //Recurse
                 walk_ports_recurse(p, name_buffer, buffer_size,
-                                   *base, data, walker, runtime, old_end);
+                                   *base, data, walker, runtime, old_end,
+                                   expand_bundles);
             }
         } else {
             if(strchr(p.name,'#')) {
-                const char *name = p.name;
-                char       *pos  = old_end;
-                while(*name != '#') *pos++ = *name++;
-                const unsigned max = atoi(name+1);
-                while(isdigit(*++name)) ;
-
-                for(unsigned i=0; i<max; ++i)
-                {
-                    char* pos_after_num = pos + sprintf(pos,"%d",i);
-                    const char* name2_2 = name;
-
-                    // append everything behind the '#' (for cases like a#N/b)
-                    while(*name2_2 && *name2_2 != ':')
-                        *pos_after_num++ = *name2_2++;
-
-                    //Apply walker function
-                    walker(&p, name_buffer, old_end, *base, data, runtime);
-                }
+                bundle_foreach(p, old_end, name_buffer, *base, data, runtime,
+                               walker, expand_bundles);
             } else {
                 //Append the path
                 scat(name_buffer, p.name);
@@ -1727,7 +1993,7 @@ void walk_ports2(const rtosc::Ports *base,
 
                     //Recurse
                     walk_ports2(p.ports, name_buffer, buffer_size,
-                            data, walker);
+                                data, walker);
                 }
             } else {
                 //Append the path
