@@ -17,6 +17,16 @@ void port_sugar()
                   "Port sugar for presets (3)", __LINE__);
     assert_str_eq("=2", presets_str+25,
                   "Port sugar for presets (4)", __LINE__);
+
+    const char* multi_str = rPresetAtMulti(false, 0, 2);
+    assert_str_eq(":default 0", multi_str,
+                  "Port sugar for presets (1)", __LINE__);
+    assert_str_eq("=false", multi_str+11,
+                  "Port sugar for presets (2)", __LINE__);
+    assert_str_eq(":default 2", multi_str+18,
+                  "Port sugar for presets (3)", __LINE__);
+    assert_str_eq("=false", multi_str+29,
+                  "Port sugar for presets (4)", __LINE__);
 }
 
 static const Ports ports = {
@@ -95,8 +105,9 @@ struct Envelope
     }
 
     std::size_t sustain, attack_rate;
-    int scale_type;
+    int scale_type; //!< 0=log, 1=lin
     size_t env_type;
+    int array[4];
 
     // constructor; sets all values to default
     Envelope() : scale_type(1), env_type(0)
@@ -111,10 +122,13 @@ struct Envelope
             case 0:
                 sustain = 30;
                 attack_rate = 40;
+                memset(array, 0, 4*sizeof(int));
                 break;
             case 1:
                 sustain = 127;
                 attack_rate = 127;
+                for(std::size_t i = 0; i < 4; ++i)
+                    array[i] = i;
                 break;
         }
     }
@@ -139,13 +153,14 @@ static const Ports envelope_ports = {
             obj->read_or_store(m, d, obj->attack_rate); }},
     rOption(scale_type, rProp(parameter) rOpt(0, logarithmic), rOpt(1, linear),
             rDefault(linear), "scale type"),
-    // array: env_type 0: 0 1 2 3
-    //        env_type 1: 0 0 0 0
-/*    {"array#4::i", rDefaultDepends(env_type),
-        r(0, ),
-        rPreset(1, 0), // bash the whole array to 0 for preset 1
-        rDefault(3), // all not yet specified values are set to 3
-        rMap(default )}*/
+    // array: env_type 0: 0 0 0 0
+    //        env_type 1: 0 1 2 3
+    rArrayI(array, 4, rDefaultDepends(env_type),
+            /*rPreset(0, 0),*/
+            rPreset(0, [4x0]),
+            rPreset(1, [0 1 2 3]), // bash the whole array to 0 for preset 1
+            rDefault(3), // all not yet specified values are set to 3
+            "some bundle"),
     // port without rDefault macros
     // must not be noted for the savefile
     {"paste:b", rProp(parameter), NULL,
@@ -181,6 +196,8 @@ void envelope_types()
     e2.attack_rate     = 127; // = envelope 1's default
     e2.env_type        =   1; // != default
     e2.scale_type      =   0; // != default
+    for(size_t i = 0; i < 4; ++i)
+        e2.array[i] = 3-i;
 
     assert_str_eq("30", get_default_value("sustain::i", envelope_ports, &e1),
                   "get default value with runtime (1)", __LINE__);
@@ -197,7 +214,8 @@ void envelope_types()
     assert_str_eq("/sustain 40",
                   get_changed_values(envelope_ports, &e1).c_str(),
                   "get changed values where none are changed", __LINE__);
-    const char* changed_e2 = "/sustain 0\n/scale_type logarithmic\n/env_type 1";
+    const char* changed_e2 = "/sustain 0\n/scale_type logarithmic\n"
+                             "/array [3 2 1 0]\n/env_type 1";
     assert_str_eq(changed_e2,
                   get_changed_values(envelope_ports, &e2).c_str(),
                   "get changed values where three are changed", __LINE__);
@@ -253,8 +271,8 @@ void envelope_types()
                               envelope_ports, &e2_restored,
                               appname.c_str(), appver);
 
-    assert_int_eq(3, rval,
-                  "load savefile from file, 3 messages read", __LINE__);
+    assert_int_eq(4, rval,
+                  "load savefile from file, 4 messages read", __LINE__);
 
     auto check_restored = [](int i1, int i2, const char* member) {
         std::string tcs = "restore "; tcs += member; tcs += " from file";
@@ -264,6 +282,10 @@ void envelope_types()
     check_restored(e2.sustain, e2_restored.sustain, "sustain value");
     check_restored(e2.attack_rate, e2_restored.attack_rate, "attack rate");
     check_restored(e2.scale_type, e2_restored.scale_type, "scale type");
+    check_restored(e2.array[0], e2_restored.array[0], "array[0]");
+    check_restored(e2.array[1], e2_restored.array[1], "array[1]");
+    check_restored(e2.array[2], e2_restored.array[2], "array[2]");
+    check_restored(e2.array[3], e2_restored.array[3], "array[3]");
     check_restored(e2.env_type, e2_restored.env_type, "envelope type");
 }
 
@@ -321,46 +343,67 @@ void savefiles()
 
     struct my_dispatcher_t : public rtosc::savefile_dispatcher_t
     {
+
+        int modify(char* portname,
+                   size_t maxargs, size_t nargs, rtosc_arg_val_t* args,
+                   bool round2)
+        {
+            int newsize = nargs;
+
+            if(round2)
+            {
+                if(rtosc_version_cmp(rtosc_filever,
+                                     rtosc_version{0, 0, 4}) < 0)
+                   /* version < 0.0.4 ? */
+                {
+                    if(rtosc_version_cmp(rtosc_filever,
+                                         rtosc_version{0, 0, 3}) < 0)
+                    {
+                        if(rtosc_version_cmp(rtosc_filever,
+                                             rtosc_version{0, 0, 2}) < 0)
+                        {
+                            {
+                                // dispatch an additional message
+                                char buffer[32];
+                                rtosc_message(buffer, 32,
+                                              "/very_old_version", "T");
+                                operator()(buffer);
+                            }
+
+                            if(nargs == 0 && maxargs >= 1)
+                            {
+                                memcpy(portname+1, "new", 3);
+                                args[0].val.i = 42;
+                                args[0].type = 'i';
+                                newsize = 1;
+                            }
+                            else // wrong argument count or not enough space
+                                newsize = abort;
+                        }
+                        else // i.e. version = 0.0.2
+                            newsize = discard;
+                    }
+                    else // i.e. version = 0.0.3
+                        newsize = abort;
+                }
+                /* for versions >= 0.0.4, we just let "/old_param" pass */
+                /* in order to get a dispatch error */
+            }
+            else {
+                // discard "/old_param" in round 1, since nothing depends on it
+                newsize = discard;
+            }
+
+            return newsize;
+        }
+
         int on_dispatch(size_t, char* portname,
                         size_t maxargs, size_t nargs, rtosc_arg_val_t* args,
                         bool round2, dependency_t dependency)
         {
-            int newsize = nargs;
-
-            if(round2 && !strcmp(portname, "/old_param"))
-            {
-                if(rtosc_version_cmp(rtosc_filever,
-                                     rtosc_version{0, 0, 3}) < 0)
-                {
-                    if(rtosc_version_cmp(rtosc_filever,
-                                         rtosc_version{0, 0, 2}) < 0)
-                    {
-                        {
-                            // dispatch an additional message
-                            char buffer[32];
-                            rtosc_message(buffer, 32, "/very_old_version", "T");
-                            operator()(buffer);
-                        }
-
-                        if(nargs == 0 && maxargs >= 1)
-                        {
-                            memcpy(portname+1, "new", 3);
-                            args[0].val.i = 42;
-                            args[0].type = 'i';
-                            newsize = 1;
-                        }
-                        else // wrong argument count or not enough space
-                            newsize = abort;
-                    }
-                    else // i.e. version = 0.0.2
-                        newsize = discard;
-                }
-                else // i.e. version >= 0.0.3
-                    newsize = abort;
-            }
-            else newsize = default_response(nargs, round2, dependency);
-
-            return newsize;
+            return (portname[1] == 'o' && !strcmp(portname, "/old_param"))
+                ? modify(portname, maxargs, nargs, args, round2)
+                : default_response(nargs, round2, dependency);
         }
     };
 
@@ -399,14 +442,26 @@ void savefiles()
                   "further parameter is being dispatched for v0.0.2", __LINE__);
 
     reset_savefile(sft);
+    // test with v0.0.3
+    // abort explicitly (see savefile dispatcher implementation)
     rval = load_from_file(MAKE_TESTFILE("v0.0.3"),
                           savefile_test_ports, &sft,
                           "savefiletest", rtosc_version {1, 2, 3},
                           &my_dispatcher);
-    assert_int_eq(-58, rval, "savefile: 1 error for v0.0.3", __LINE__);
+    assert_int_eq(-59, rval, "savefile: 1 error for v0.0.3", __LINE__);
     assert_int_eq(123, sft.further_param,
                   "no further parameter is being dispatched for v0.0.3",
                   __LINE__);
+    // test with v0.0.4
+    // abort implicitly (the port is unknown)
+    rval = load_from_file(MAKE_TESTFILE("v0.0.4"),
+                          savefile_test_ports, &sft,
+                          "savefiletest", rtosc_version {1, 2, 3},
+                          &my_dispatcher);
+    assert_int_eq(-59, rval, "savefile: 1 error for v0.0.4", __LINE__);
+    assert_int_eq(123, sft.further_param,
+                  "no further parameter is being dispatched for v0.0.4",
+                  __LINE__);
 
 #undef MAKE_TESTFILE
 }
