@@ -18,12 +18,14 @@
 #include "../Misc/Util.h"
 #include "SVFilter.h"
 
-#define errx(...)
-#define warnx(...)
+#define errx(...) {}
+#define warnx(...) {}
 #ifndef errx
 #include <err.h>
 #endif
 
+namespace zyn {
+
 SVFilter::SVFilter(unsigned char Ftype, float Ffreq, float Fq,
                    unsigned char Fstages, unsigned int srate, int bufsize)
     :Filter(srate, bufsize),
@@ -229,3 +231,5 @@ void SVFilter::filterout(float *smp)
     for(int i = 0; i < buffersize; ++i)
         smp[i] *= outgain;
 }
+
+}
