@@ -17,6 +17,8 @@
 #include "../globals.h"
 #include "Filter.h"
 
+namespace zyn {
+
 class SVFilter:public Filter
 {
     public:
@@ -67,4 +69,6 @@ class SVFilter:public Filter
         bool needsinterpolation, firsttime;
 };
 
+}
+
 #endif
