@@ -19,6 +19,8 @@
 
 #define MAX_CHORUS_DELAY 250.0f //ms
 
+namespace zyn {
+
 /**Chorus and Flange effects*/
 class Chorus:public Effect
 {
@@ -95,4 +97,6 @@ class Chorus:public Effect
         float getdelay(float xlfo);
 };
 
+}
+
 #endif
