@@ -40,7 +40,8 @@ rtosc::Ports Distorsion::ports = {
     rEffParVol(rDefault(127), rPresetsAt(2, 64, 64)),
     rEffParPan(),
     rEffPar(Plrcross, 2, rShort("l/r"), rDefault(35), "Left/Right Crossover"),
-    rEffPar(Pdrive,   3, rShort("drive"), rPresets(56, 29, 75, 85, 63, 88),
+    rEffPar(Pdrive,   3, rShort("drive"),
+            rLinear(0, 127), rPresets(56, 29, 75, 85, 63, 88),
             "Input amplification"),
     rEffPar(Plevel,   4, rShort("output"), rPresets(70, 75, 80, 62, 75, 75),
             "Output amplification"),
