@@ -16,6 +16,8 @@
 
 #include "Effect.h"
 
+namespace zyn {
+
 /**Distortion Effect*/
 class Distorsion:public Effect
 {
@@ -50,4 +52,6 @@ class Distorsion:public Effect
         class AnalogFilter * lpfl, *lpfr, *hpfl, *hpfr;
 };
 
+}
+
 #endif
