@@ -17,6 +17,8 @@
 #include "Effect.h"
 #include "../Misc/Stereo.h"
 
+namespace zyn {
+
 /**Echo Effect*/
 class Echo:public Effect
 {
@@ -94,4 +96,6 @@ class Echo:public Effect
         Stereo<int> ndelta;
 };
 
+}
+
 #endif
