@@ -19,6 +19,8 @@
 #define REV_COMBS 8
 #define REV_APS 4
 
+namespace zyn {
+
 /**Creates Reverberation Effects*/
 class Reverb:public Effect
 {
@@ -83,4 +85,6 @@ class Reverb:public Effect
         class AnalogFilter * lpf, *hpf; //filters
 };
 
+}
+
 #endif
