@@ -49,7 +49,7 @@ namespace zyn {
 
 static const Ports sysefxPort =
 {
-    {"part#" STRINGIFY(NUM_MIDI_PARTS) "::i", rProp(parameter)
+    {"part#" STRINGIFY(NUM_MIDI_PARTS) "::i", rProp(parameter) rDefault([0...])
         rDoc("gain on part to sysefx routing"), 0,
         [](const char *m, RtData&d)
         {
@@ -76,8 +76,8 @@ static const Ports sysefxPort =
             Master &mast = *(Master*)d.obj;
 
             if(rtosc_narguments(m)) {
-                mast.setPsysefxvol(ind2, ind1, rtosc_argument(m,0).i);
-                d.broadcast(d.loc, "i", mast.Psysefxvol[ind1][ind2]);
+             mast.setPsysefxvol(ind2, ind1, rtosc_argument(m,0).i);
+             d.broadcast(d.loc, "i", mast.Psysefxvol[ind1][ind2]);
             } else
                 d.reply(d.loc, "i", mast.Psysefxvol[ind1][ind2]);
         }}
@@ -85,8 +85,8 @@ static const Ports sysefxPort =
 
 static const Ports sysefsendto =
 {
-    {"to#" STRINGIFY(NUM_SYS_EFX) "::i",
-        rProp(parameter) rDoc("sysefx to sysefx routing gain"), 0, [](const char *m, RtData&d)
+    {"to#" STRINGIFY(NUM_SYS_EFX) "::i", rProp(parameter)  rDefault([0...])
+        rDoc("sysefx to sysefx routing gain"), 0, [](const char *m, RtData&d)
         {
             //same workaround as before
             //go backto the '/'
@@ -131,7 +131,7 @@ static int get_next_int(const char *msg)
 }
 
 static const Ports mapping_ports = {
-    {"offset::f", rProp(parameter) rShort("off") rLinear(-50, 50) rMap(unit, percent), 0,
+    {"offset::f", rProp(parameter) rDefault(0) rShort("off") rLinear(-50, 50) rMap(unit, percent), 0,
         rBegin;
         int slot = d.idx[1];
         int param = d.idx[0];
@@ -142,7 +142,7 @@ static const Ports mapping_ports = {
         } else
             d.reply(d.loc, "f", a.getSlotSubOffset(slot, param));
         rEnd},
-    {"gain::f", rProp(parameter) rShort("gain") rLinear(-200, 200) rMap(unit, percent), 0,
+    {"gain::f", rProp(parameter) rDefault(100) rShort("gain") rLinear(-200, 200) rMap(unit, percent), 0,
         rBegin;
         int slot = d.idx[1];
         int param = d.idx[0];
@@ -178,7 +178,7 @@ static const Ports auto_param_ports = {
         int param = d.idx[0];
         d.reply(d.loc, "s", a.slots[slot].automations[param].param_path);
         rEnd},
-    {"clear:", 0, 0,
+    {"clear:", rDoc("Clear automation param"), 0,
         rBegin;
         int slot = d.idx[1];
         int param = d.idx[0];
@@ -186,6 +186,7 @@ static const Ports auto_param_ports = {
         rEnd},
     {"mapping/", 0, &mapping_ports,
         rBegin;
+        (void) a;
         SNIP;
         mapping_ports.dispatch(msg, d);
         rEnd},
@@ -219,7 +220,7 @@ static const Ports slot_ports = {
     //                              rtosc_argument(msg, 1).s,
     //                              rtosc_argument(msg, 2).T);
     //    rEnd},
-    {"value::f", rProp(parameter) rMap(default, 0.5) rLinear(0, 1) rDoc("Access current value in slot 'i' (0..1)"), 0,
+    {"value::f", rProp(no learn) rProp(parameter) rMap(default, 0.5) rLinear(0, 1) rDoc("Access current value in slot 'i' (0..1)"), 0,
         rBegin;
         int num = d.idx[0];
         if(!strcmp("f",rtosc_argument_string(msg))) {
@@ -260,7 +261,7 @@ static const Ports slot_ports = {
         int slot = d.idx[0];
         d.reply(d.loc, "i", a.slots[slot].learning);
         rEnd},
-    {"clear:", 0, 0,
+    {"clear:", rDoc("Clear automation slot"), 0,
         rBegin;
         int slot = d.idx[0];
         a.clearSlot(slot);
@@ -306,7 +307,7 @@ static const Ports automate_ports = {
         slot_ports.dispatch(msg, d);
         d.pop_index();
         rEnd},
-    {"clear", 0, 0,
+    {"clear", rDoc("Clear all automation slots"), 0,
         rBegin;
         for(int i=0; i<a.nslots; ++i)
             a.clearSlot(i);
@@ -366,7 +367,7 @@ static const Ports master_ports = {
     rArrayOption(Pinsparts, NUM_INS_EFX, rOpt(-2, Master), rOpt(-1, Off),
                  rOptions(Part1, Part2, Part3, Part4, Part5, Part6,
                           Part7, Part8, Part9, Part10, Part11, Part12,
-                          Part13, Part14, Part15, Part16) rDefault(Off),
+                          Part13, Part14, Part15, Part16) rDefault([Off ...]),
                  "Part to insert part onto"),
     {"Pkeyshift::i", rShort("key shift") rProp(parameter) rLinear(0,127)
         rDefault(64) rDoc("Global Key Shift"), 0, [](const char *m, RtData&d) {
@@ -487,7 +488,7 @@ static const Ports master_ports = {
             d.obj = (void*)&((Master*)d.obj)->automate;
             automate_ports.dispatch(msg, d);
             }},
-    {"close-ui:", rDoc("Request to close any connection named \"GUI\""), 0,
+    {"close-ui:", rDoc("Request to close the unique connection named \"GUI\""), 0,
         [](const char *, RtData &d) {
        d.reply("/close-ui", "");}},
     {"add-rt-memory:bi", rProp(internal) rDoc("Add Additional Memory To RT MemPool"), 0,
@@ -664,7 +665,6 @@ void Master::loadAutomation(XMLwrapper &xml, rtosc::AutomationMgr &midi)
             auto &slot = midi.slots[i];
             if(xml.enterbranch("slot", i)) {
                 for(int j=0; j<midi.per_slot; ++j) {
-                    auto &au = slot.automations[j];
                     if(xml.enterbranch("automation", j)) {
                         float gain       = 1.0;
                         float offset     = 0.0;
@@ -750,25 +750,86 @@ Master::Master(const SYNTH_T &synth_, Config* config)
     mastercb_ptr = 0;
 }
 
-void Master::applyOscEvent(const char *msg)
+bool Master::applyOscEvent(const char *msg, float *outl, float *outr,
+                           bool offline, bool nio, DataObj& d, int msg_id)
+{
+    if(!strcmp(msg, "/load-master")) {
+        Master *this_master = this;
+        Master *new_master  = *(Master**)rtosc_argument(msg, 0).b.data;
+        if(!offline)
+            new_master->AudioOut(outl, outr);
+        if(nio)
+            Nio::masterSwap(new_master);
+        if (hasMasterCb())
+            mastercb(mastercb_ptr, new_master);
+        bToU->write("/free", "sb", "Master", sizeof(Master*), &this_master);
+        return false;
+    } else if(!strcmp(msg, "/switch-master")) {
+        // if the other stuff from load-master is needed optionally
+        // (currently, it is not needed anywhere)
+        // add booleans to the parameters of "/switch-master"
+        Master *new_master  = *(Master**)rtosc_argument(msg, 0).b.data;
+        if (hasMasterCb())
+            mastercb(mastercb_ptr, new_master);
+        return false;
+    }
+
+    //XXX yes, this is not realtime safe, but it is useful...
+    if(strcmp(msg, "/get-vu") && false) {
+        fprintf(stdout, "%c[%d;%d;%dm", 0x1B, 0, 5 + 30, 0 + 40);
+        if(msg_id > 0)
+            fprintf(stdout, "backend[%d]: '%s'<%s>\n", msg_id, msg,
+                    rtosc_argument_string(msg));
+        else
+            fprintf(stdout, "backend[*]: '%s'<%s>\n", msg,
+                    rtosc_argument_string(msg));
+        fprintf(stdout, "%c[%d;%d;%dm", 0x1B, 0, 7 + 30, 0 + 40);
+    }
+
+    ports.dispatch(msg, d, true);
+
+    if(!d.matches) {
+        //workaround for requesting voice status
+        int a=0, b=0, c=0;
+        char e=0;
+        if(4 == sscanf(msg, "/part%d/kit%d/adpars/VoicePar%d/Enable%c", &a, &b, &c, &e)) {
+            d.reply(msg, "F");
+            d.matches++;
+        }
+    }
+    if(!d.matches && !d.forwarded) {// && !ports.apropos(msg)) {
+        fprintf(stderr, "%c[%d;%d;%dm", 0x1B, 1, 7 + 30, 0 + 40);
+        fprintf(stderr, "Unknown address<BACKEND:%s> '%s:%s'\n",
+                offline ? "offline" : "online",
+                uToB->peak(),
+                rtosc_argument_string(uToB->peak()));
+        fprintf(stderr, "%c[%d;%d;%dm", 0x1B, 0, 7 + 30, 0 + 40);
+    }
+    else if(d.forwarded)
+        bToU->raw_write(msg);
+
+    if(d.matches == 0 && !d.forwarded)
+        fprintf(stderr, "Unknown path '%s:%s'\n", msg, rtosc_argument_string(msg));
+    if(d.forwarded)
+        bToU->raw_write(msg);
+
+    return true;
+}
+
+bool Master::applyOscEvent(const char *msg, float *outl, float *outr,
+                           bool offline, bool nio, int msg_id)
 {
     char loc_buf[1024];
     DataObj d{loc_buf, 1024, this, bToU};
     memset(loc_buf, 0, sizeof(loc_buf));
     d.matches = 0;
 
-    if(strcmp(msg, "/get-vu") && false) {
-        fprintf(stdout, "%c[%d;%d;%dm", 0x1B, 0, 5 + 30, 0 + 40);
-        fprintf(stdout, "backend[*]: '%s'<%s>\n", msg,
-                rtosc_argument_string(msg));
-        fprintf(stdout, "%c[%d;%d;%dm", 0x1B, 0, 7 + 30, 0 + 40);
-    }
+    return applyOscEvent(msg, outl, outr, offline, nio, d, msg_id);
+}
 
-    ports.dispatch(msg, d, true);
-    if(d.matches == 0 && !d.forwarded)
-        fprintf(stderr, "Unknown path '%s:%s'\n", msg, rtosc_argument_string(msg));
-    if(d.forwarded)
-        bToU->raw_write(msg);
+bool Master::applyOscEvent(const char *msg, bool nio, int msg_id)
+{
+    return applyOscEvent(msg, NULL, NULL, true, nio, msg_id);
 }
 
 void Master::defaults()
@@ -965,6 +1026,18 @@ void Master::setMasterChangedCallback(void(*cb)(void*,Master*), void *ptr)
     mastercb_ptr = ptr;
 }
 
+void Master::copyMasterCbTo(Master *dest)
+{
+    dest->mastercb     = mastercb;
+    dest->mastercb_ptr = mastercb_ptr;
+}
+
+bool Master::hasMasterCb() const
+{
+    return !!mastercb;
+}
+
+
 #if 0
 template <class T>
 struct def_skip
@@ -1022,52 +1095,17 @@ int msg_id=0;
 
 bool Master::runOSC(float *outl, float *outr, bool offline)
 {
-    //Handle user events TODO move me to a proper location
+    //Handle user events
     char loc_buf[1024];
     DataObj d{loc_buf, 1024, this, bToU};
     memset(loc_buf, 0, sizeof(loc_buf));
+
     int events = 0;
-    while(uToB && uToB->hasNext() && events < 100) {
+    for(; uToB && uToB->hasNext() && events < 100; ++msg_id, ++events)
+    {
         const char *msg = uToB->read();
-
-        if(!strcmp(msg, "/load-master")) {
-            Master *this_master = this;
-            Master *new_master  = *(Master**)rtosc_argument(msg, 0).b.data;
-            if(!offline)
-                new_master->AudioOut(outl, outr);
-            Nio::masterSwap(new_master);
-            if (mastercb)
-                mastercb(mastercb_ptr, new_master);
-            bToU->write("/free", "sb", "Master", sizeof(Master*), &this_master);
+        if(! applyOscEvent(msg, outl, outr, offline, true, d, msg_id) )
             return false;
-        }
-
-        //XXX yes, this is not realtime safe, but it is useful...
-        if(strcmp(msg, "/get-vu") && false) {
-            fprintf(stdout, "%c[%d;%d;%dm", 0x1B, 0, 5 + 30, 0 + 40);
-            fprintf(stdout, "backend[%d]: '%s'<%s>\n", msg_id++, msg,
-                    rtosc_argument_string(msg));
-            fprintf(stdout, "%c[%d;%d;%dm", 0x1B, 0, 7 + 30, 0 + 40);
-        }
-        ports.dispatch(msg, d, true);
-        events++;
-        if(!d.matches) {
-            //workaround for requesting voice status
-            int a=0, b=0, c=0;
-            char e=0;
-            if(4 == sscanf(msg, "/part%d/kit%d/adpars/VoicePar%d/Enable%c", &a, &b, &c, &e)) {
-                d.reply(msg, "F");
-                d.matches++;
-            }
-        }
-        if(!d.matches) {// && !ports.apropos(msg)) {
-            fprintf(stderr, "%c[%d;%d;%dm", 0x1B, 1, 7 + 30, 0 + 40);
-            fprintf(stderr, "Unknown address<BACKEND:%s> '%s:%s'\n",
-                    offline ? "offline" : "online",
-                    uToB->peak(),
-                    rtosc_argument_string(uToB->peak()));
-            fprintf(stderr, "%c[%d;%d;%dm", 0x1B, 0, 7 + 30, 0 + 40);
-        }
     }
 
     if(automate.damaged) {
@@ -1618,25 +1656,40 @@ char* Master::getXMLData()
     return xml.getXMLdata();
 }
 
-int Master::saveOSC(const char *filename)
+// this is being called as a "read only op" directly by the MiddleWare thread;
+// note that the Master itself is frozen
+int Master::saveOSC(const char *filename, master_dispatcher_t* dispatcher,
+                    Master* master2)
 {
     std::string savefile = rtosc::save_to_file(ports, this,
                                                "ZynAddSubFX",
                                                version_in_rtosc_fmt());
 
-    zyn::Config config;
-    zyn::SYNTH_T* synth = new zyn::SYNTH_T;
-    synth->buffersize = 256;
-    synth->samplerate = 48000;
-    synth->alias();
+    // load the savefile string into another master to compare the results
+    // between the original and the savefile-loaded master
+    // this requires a temporary master switch
+    dispatcher->updateMaster(master2);
 
-    zyn::Master master2(*synth, &config);
-    int rval = master2.loadOSCFromStr(savefile.c_str());
+    int rval = master2->loadOSCFromStr(savefile.c_str(), dispatcher);
 
+    // The above call is done by this thread (i.e. the MiddleWare thread), but
+    // it sends messages to master2 in order to load the values
+    // We need to wait until savefile has been loaded into master2
+    int i;
+    for(i = 0; i < 20 && master2->uToB->hasNext(); ++i)
+        usleep(50000);
+    if(i >= 20) // >= 1 second?
+    {
+        // Master failed to fetch its messages
+        rval = -1;
+    }
+    printf("Saved in less than %d ms.\n", 50*i);
+
+    dispatcher->updateMaster(this);
 
     if(rval < 0)
     {
-        std::cerr << "invalid savefile!" << std::endl;
+        std::cerr << "invalid savefile (or a backend error)!" << std::endl;
         std::cerr << "complete savefile:" << std::endl;
         std::cerr << savefile << std::endl;
         std::cerr << "first entry that could not be parsed:" << std::endl;
@@ -1654,19 +1707,23 @@ int Master::saveOSC(const char *filename)
     else
     {
         char* xml = getXMLData(),
-            * xml2 = master2.getXMLData();
+            * xml2 = master2->getXMLData();
 
         rval = strcmp(xml, xml2) ? -1 : 0;
 
         if(rval == 0)
         {
-            if(filename)
+            if(filename && *filename)
             {
                 std::ofstream ofs(filename);
                 ofs << savefile;
             }
-            else if(!filename)
+            else {
+                std::cout << "The savefile content follows" << std::endl;
+                std::cout << "---->8----" << std::endl;
                 std::cout << savefile << std::endl;
+                std::cout << "---->8----" << std::endl;
+            }
         }
         else
         {
@@ -1676,6 +1733,7 @@ int Master::saveOSC(const char *filename)
             std::ofstream tmp1("tmp1.txt"), tmp2("tmp2.txt");
             tmp1 << xml;
             tmp2 << xml2;
+            rval = -1;
         }
 
         free(xml);
@@ -1684,11 +1742,13 @@ int Master::saveOSC(const char *filename)
     return rval;
 }
 
-int Master::loadOSCFromStr(const char *filename)
+int Master::loadOSCFromStr(const char *file_content,
+                           savefile_dispatcher_t* dispatcher)
 {
-    return rtosc::load_from_file(filename,
+    return rtosc::load_from_file(file_content,
                                  ports, this,
-                                 "ZynAddSubFX", version_in_rtosc_fmt());
+                                 "ZynAddSubFX", version_in_rtosc_fmt(),
+                                 dispatcher);
 }
 
 string loadfile(string fname)
@@ -1699,9 +1759,9 @@ string loadfile(string fname)
     return str;
 }
 
-int Master::loadOSC(const char *filename)
+int Master::loadOSC(const char *filename, savefile_dispatcher_t* dispatcher)
 {
-    int rval = loadOSCFromStr(loadfile(filename).c_str());
+    int rval = loadOSCFromStr(loadfile(filename).c_str(), dispatcher);
     return rval < 0 ? rval : 0;
 }
 
