@@ -18,6 +18,7 @@
 #include <iostream>
 #include <dirent.h>
 #include <sys/stat.h>
+#include <mutex>
 
 #include <rtosc/undo-history.h>
 #include <rtosc/thread-link.h>
@@ -207,17 +208,31 @@ void preparePadSynth(string path, PADnoteParameters *p, rtosc::RtData &d)
     assert(!path.empty());
     path += "sample";
 
-    unsigned max = 0;
-    p->sampleGenerator([&max,&path,&d]
-            (unsigned N, PADnoteParameters::Sample &s)
-            {
-            max = max<N ? N : max;
-            //printf("sending info to '%s'\n", (path+to_s(N)).c_str());
-            d.chain((path+to_s(N)).c_str(), "ifb",
-                s.size, s.basefreq, sizeof(float*), &s.smp);
-            }, []{return false;});
+#ifdef WIN32
+    unsigned num = p->sampleGenerator([&path,&d]
+                       (unsigned N, PADnoteParameters::Sample &s)
+                       {
+                           //printf("sending info to '%s'\n",
+                           //       (path+to_s(N)).c_str());
+                           d.chain((path+to_s(N)).c_str(), "ifb",
+                                   s.size, s.basefreq, sizeof(float*), &s.smp);
+                       }, []{return false;}, 1);
+#else
+    std::mutex rtdata_mutex;
+    unsigned num = p->sampleGenerator([&rtdata_mutex, &path,&d]
+                       (unsigned N, PADnoteParameters::Sample &s)
+                       {
+                           //printf("sending info to '%s'\n",
+                           //       (path+to_s(N)).c_str());
+                           rtdata_mutex.lock();
+                           d.chain((path+to_s(N)).c_str(), "ifb",
+                                   s.size, s.basefreq, sizeof(float*), &s.smp);
+                           rtdata_mutex.unlock();
+                       }, []{return false;});
+#endif
+
     //clear out unused samples
-    for(unsigned i = max+1; i < PAD_MAX_SAMPLES; ++i) {
+    for(unsigned i = num; i < PAD_MAX_SAMPLES; ++i) {
         d.chain((path+to_s(i)).c_str(), "ifb",
                 0, 440.0f, sizeof(float*), NULL);
     }
@@ -404,6 +419,22 @@ namespace Nio
 
 
 /* Implementation */
+
+class mw_dispatcher_t : public master_dispatcher_t
+{
+    MiddleWare* mw;
+    bool do_dispatch(const char *msg) override
+    {
+        mw->transmitMsg(msg);
+        return true; // we cannot yet say if the port matched
+                     // we will query the Master after everything will be done
+    }
+    void vUpdateMaster(Master* m) { mw->switchMaster(m); }
+
+public:
+    mw_dispatcher_t(MiddleWare* mw) : mw(mw) {}
+};
+
 class MiddleWareImpl
 {
     public:
@@ -429,7 +460,8 @@ public:
 
     void savePart(int npart, const char *filename)
     {
-        //Copy is needed as filename WILL get trashed during the rest of the run
+        // Due to a possible bug in ThreadLink, filename may get trashed when
+        // the read-only operation writes to the buffer again. Copy to string:
         std::string fname = filename;
         //printf("saving part(%d,'%s')\n", npart, filename);
         doReadOnlyOp([this,fname,npart](){
@@ -528,16 +560,28 @@ public:
     }
 
     //Well, you don't get much crazier than changing out all of your RT
-    //structures at once... TODO error handling
-    void loadMaster(const char *filename)
+    //structures at once...
+    int loadMaster(const char *filename, bool osc_format = false)
     {
         Master *m = new Master(synth, config);
         m->uToB = uToB;
         m->bToU = bToU;
+
         if(filename) {
-            if ( m->loadXML(filename) ) {
-                delete m;
-                return;
+            if(osc_format)
+            {
+                mw_dispatcher_t dispatcher(parent);
+                if( m->loadOSC(filename, &dispatcher) < 0 ) {
+                    delete m;
+                    return -1;
+                }
+            }
+            else
+            {
+                if ( m->loadXML(filename) ) {
+                    delete m;
+                    return -1;
+                }
             }
             m->applyparameters();
         }
@@ -550,6 +594,38 @@ public:
         //Give it to the backend and wait for the old part to return for
         //deallocation
         parent->transmitMsg("/load-master", "b", sizeof(Master*), &m);
+        return 0;
+    }
+
+    int saveMaster(const char *filename, bool osc_format = false)
+    {
+        int res;
+        if(osc_format)
+        {
+            mw_dispatcher_t dispatcher(parent);
+
+            // allocate an "empty" master
+            // after the savefile will have been saved, it will be loaded into this
+            // dummy master, and then the two masters will be compared
+            zyn::Config config;
+            zyn::SYNTH_T* synth = new zyn::SYNTH_T;
+            synth->buffersize = master->synth.buffersize;
+            synth->samplerate = master->synth.samplerate;
+            synth->alias();
+            zyn::Master master2(*synth, &config);
+            master->copyMasterCbTo(&master2);
+            master2.frozenState = true;
+
+            doReadOnlyOp([this,filename,&dispatcher,&master2,&res](){
+                             res = master->saveOSC(filename, &dispatcher,
+                                                   &master2);});
+        }
+        else // xml format
+        {
+            doReadOnlyOp([this,filename,&res](){
+                             res = master->saveXML(filename);});
+        }
+        return res;
     }
 
     void loadXsz(const char *filename, rtosc::RtData &d)
@@ -776,11 +852,19 @@ class MwDataObj:public rtosc::RtData
                 reply(buffer);
             }
         }
+        //! In the case of MiddleWare, "reply" always means sending back to
+        //! the front-end. If a message from the back-end gets "replied", this
+        //! only means that its counterpart has been sent from the front-end via
+        //! MiddleWare to the backend, so the reply has to go back to the
+        //! front-end. The back-end itself usually doesn't ask things, so it
+        //! will not get replies.
         virtual void reply(const char *msg) override{
             mwi->sendToCurrentRemote(msg);
-        };
-        //virtual void broadcast(const char *path, const char *args, ...){(void)path;(void)args;};
-        //virtual void broadcast(const char *msg){(void)msg;};
+        }
+
+        virtual void broadcast(const char *msg) override {
+            mwi->broadcastToRemote(msg);
+        }
 
         virtual void chain(const char *msg) override
         {
@@ -889,13 +973,28 @@ extern const rtosc::Ports bankPorts;
 const rtosc::Ports bankPorts = {
     {"rescan:", 0, 0,
         rBegin;
+        impl.bankpos = 0;
         impl.rescanforbanks();
         //Send updated banks
         int i = 0;
         for(auto &elm : impl.banks)
             d.reply("/bank/bank_select", "iss", i++, elm.name.c_str(), elm.dir.c_str());
         d.reply("/bank/bank_select", "i", impl.bankpos);
+        if (i > 0) {
+            impl.loadbank(impl.banks[0].dir);
 
+            //Reload bank slots
+            for(int i=0; i<BANK_SIZE; ++i) {
+                d.reply("/bankview", "iss",
+                    i, impl.ins[i].name.c_str(),
+                    impl.ins[i].filename.c_str());
+            }
+        } else {
+            //Clear all bank slots
+            for(int i=0; i<BANK_SIZE; ++i) {
+                d.reply("/bankview", "iss", i, "", "");
+            }
+        }
         rEnd},
     {"bank_list:", 0, 0,
         rBegin;
@@ -1043,8 +1142,8 @@ const rtosc::Ports bankPorts = {
         rBegin;
         auto res = impl.search(rtosc_argument(msg, 0).s);
 #define MAX_SEARCH 300
-        char res_type[MAX_SEARCH+1] = {0};
-        rtosc_arg_t res_dat[MAX_SEARCH] = {0};
+        char res_type[MAX_SEARCH+1] = {};
+        rtosc_arg_t res_dat[MAX_SEARCH] = {};
         for(unsigned i=0; i<res.size() && i<MAX_SEARCH; ++i) {
             res_type[i]  = 's';
             res_dat[i].s = res[i].c_str();
@@ -1056,8 +1155,8 @@ const rtosc::Ports bankPorts = {
         rBegin;
         auto res = impl.blist(rtosc_argument(msg, 0).s);
 #define MAX_SEARCH 300
-        char res_type[MAX_SEARCH+1] = {0};
-        rtosc_arg_t res_dat[MAX_SEARCH] = {0};
+        char res_type[MAX_SEARCH+1] = {};
+        rtosc_arg_t res_dat[MAX_SEARCH] = {};
         for(unsigned i=0; i<res.size() && i<MAX_SEARCH; ++i) {
             res_type[i]  = 's';
             res_dat[i].s = res[i].c_str();
@@ -1089,6 +1188,42 @@ const rtosc::Ports bankPorts = {
 #define STRINGIFY(a) STRINGIFY2(a)
 #endif
 
+/*
+ * common snoop port callbacks
+ */
+template<bool osc_format>
+void load_cb(const char *msg, RtData &d)
+{
+    MiddleWareImpl &impl = *((MiddleWareImpl*)d.obj);
+    const char *file = rtosc_argument(msg, 0).s;
+    uint64_t request_time = 0;
+    if(rtosc_narguments(msg) > 1)
+        request_time = rtosc_argument(msg, 1).t;
+
+    if(!impl.loadMaster(file, osc_format)) { // return-value 0 <=> OK
+        d.broadcast("/damage", "s", "/");
+        d.broadcast(d.loc, "stT", file, request_time);
+    }
+    else
+        d.broadcast(d.loc, "stF", file, request_time);
+}
+
+template<bool osc_format>
+void save_cb(const char *msg, RtData &d)
+{
+    MiddleWareImpl &impl = *((MiddleWareImpl*)d.obj);
+    // Due to a possible bug in ThreadLink, filename may get trashed when
+    // the read-only operation writes to the buffer again. Copy to string:
+    const string file = rtosc_argument(msg, 0).s;
+    uint64_t request_time = 0;
+    if(rtosc_narguments(msg) > 1)
+        request_time = rtosc_argument(msg, 1).t;
+
+    int res = impl.saveMaster(file.c_str(), osc_format);
+    d.broadcast(d.loc, (res == 0) ? "stT" : "stF",
+                file.c_str(), request_time);
+}
+
 /*
  * BASE/part#/kititem#
  * BASE/part#/kit#/adpars/voice#/oscil/\*
@@ -1197,14 +1332,8 @@ static rtosc::Ports middwareSnoopPorts = {
         const char *file = rtosc_argument(msg, 0).s;
         impl.loadKbm(file, d);
         rEnd},
-    {"save_xmz:s", 0, 0,
-        rBegin;
-        const char *file = rtosc_argument(msg, 0).s;
-        //Copy is needed as filename WILL get trashed during the rest of the run
-        impl.doReadOnlyOp([&impl,file](){
-                int res = impl.master->saveXML(file);
-                (void)res;});
-        rEnd},
+    {"save_xmz:s:st", 0, 0, save_cb<false>},
+    {"save_osc:s:st", 0, 0, save_cb<true>},
     {"save_xiz:is", 0, 0,
         rBegin;
         const int   part_id = rtosc_argument(msg,0).i;
@@ -1287,12 +1416,8 @@ static rtosc::Ports middwareSnoopPorts = {
         const string save_loc  = save_dir + "/" + save_file;
         remove(save_loc.c_str());
         rEnd},
-    {"load_xmz:s", 0, 0,
-        rBegin;
-        const char *file = rtosc_argument(msg, 0).s;
-        impl.loadMaster(file);
-        d.reply("/damage", "s", "/");
-        rEnd},
+    {"load_xmz:s:st", 0, 0, load_cb<false>},
+    {"load_osc:s:st", 0, 0, load_cb<true>},
     {"reset_master:", 0, 0,
         rBegin;
         impl.loadMaster(NULL);
@@ -1354,7 +1479,7 @@ static rtosc::Ports middwareSnoopPorts = {
     //    //cc-id, path, min, max
 //#define MAX_MIDI 32
     //    rtosc_arg_t args[MAX_MIDI*4];
-    //    char        argt[MAX_MIDI*4+1] = {0};
+    //    char        argt[MAX_MIDI*4+1] = {};
     //    int j=0;
     //    for(unsigned i=0; i<key.size() && i<MAX_MIDI; ++i) {
     //        auto val = midi.inv_map[key[i]];
@@ -1561,7 +1686,7 @@ void MiddleWareImpl::doReadOnlyOp(std::function<void()> read_only_fn)
     int tries = 0;
     while(tries++ < 10000) {
         if(!bToU->hasNext()) {
-            usleep(500);
+            os_usleep(500);
             continue;
         }
         const char *msg = bToU->read();
@@ -1674,7 +1799,7 @@ bool MiddleWareImpl::doReadOnlyOpNormal(std::function<void()> read_only_fn, bool
     int tries = 0;
     while(tries++ < 2000) {
         if(!bToU->hasNext()) {
-            usleep(500);
+            os_usleep(500);
             continue;
         }
         const char *msg = bToU->read();
@@ -2120,4 +2245,22 @@ PresetsStore& MiddleWare::getPresetsStore()
     return impl->presetsstore;
 }
 
+void MiddleWare::switchMaster(Master* new_master)
+{
+    // this function is kept similar to loadMaster
+    assert(impl->master->frozenState);
+
+    new_master->uToB = impl->uToB;
+    new_master->bToU = impl->bToU;
+    impl->updateResources(new_master);
+    impl->master = new_master;
+
+    if(impl->master->hasMasterCb())
+    {
+        // inform the realtime thread about the switch
+        // this will be done by calling the mastercb
+        transmitMsg("/switch-master", "b", sizeof(Master*), &new_master);
+    }
+}
+
 }
