@@ -49,9 +49,13 @@ static const Ports partPorts = {
     rRecur(ctl,       "Controller"),
     rParamZyn(partno, rProp(internal),
               "How many parts are before this in the Master"),
+#undef  rChangeCb
+#define rChangeCb if(obj->Penabled == false) obj->AllNotesOff();
     rToggle(Penabled, rShort("enable"), rDefaultDepends(partno),
             rPresets(true), rDefault(false), "Part enable"),
 #undef rChangeCb
+#define rChangeCb
+#undef rChangeCb
 #define rChangeCb obj->setPvolume(obj->Pvolume);
     rParamZyn(Pvolume, rShort("Vol"), rDefault(96),"Part Volume"),
 #undef rChangeCb
@@ -88,9 +92,10 @@ static const Ports partPorts = {
         "Instrument comments"),
     rString(Pname, PART_MAX_NAME_LEN, rDefault(""), "User specified label"),
     rArrayI(Pefxroute, NUM_PART_EFX,
-            rOptions(Next Effect,Part Out,Dry Out), rDefaultId(Next Effect),
+            rOptions(Next Effect,Part Out,Dry Out),
+            ":default\0=[\"Next Effect\"S...]\0",
             "Effect Routing"),
-    rArrayT(Pefxbypass, NUM_PART_EFX, rDefault(false),
+    rArrayT(Pefxbypass, NUM_PART_EFX, rDefault([false...]),
         "If an effect is bypassed"),
     {"captureMin:", rDoc("Capture minimum valid note"), NULL,
         [](const char *, RtData &r)
@@ -180,7 +185,7 @@ static const Ports kitPorts = {
     rParamZyn(Pminkey, rDefault(0),  "Kit item min key"),
     rParamZyn(Pmaxkey, rDefault(127)  "Kit item max key"),
     rToggle(Padenabled, rDefaultDepends(firstkit),
-            rPreset(true, true), rPreset(false, false)
+            rPreset(true, true), rPreset(false, false),
             "ADsynth enable"),
     rToggle(Psubenabled, rDefault(false), "SUBsynth enable"),
     rToggle(Ppadenabled, rDefault(false), "PADsynth enable"),
