@@ -15,6 +15,8 @@
 #define RECORDER_H
 #include <string>
 
+namespace zyn {
+
 struct SYNTH_T;
 /**Records sound to a file*/
 class Recorder
@@ -43,4 +45,6 @@ class Recorder
         const SYNTH_T &synth;
 };
 
+}
+
 #endif
