@@ -21,6 +21,8 @@
 #ifndef XML_WRAPPER_H
 #define XML_WRAPPER_H
 
+namespace zyn {
+
 class XmlAttr
 {
     public:
@@ -285,4 +287,6 @@ public:
         version_type _fileversion;
 };
 
+}
+
 #endif
