@@ -26,7 +26,10 @@
 
 #include "JackMultiEngine.h"
 
-extern MiddleWare *middleware;
+extern zyn::MiddleWare *middleware;
+
+namespace zyn {
+
 using std::string;
 
 struct jack_multi
@@ -170,3 +173,5 @@ void JackMultiEngine::Stop()
 
     impl->running = false;
 }
+
+}
