@@ -15,6 +15,8 @@
 
 #include "AudioOut.h"
 
+namespace zyn {
+
 class JackMultiEngine:public AudioOut
 {
     public:
@@ -34,4 +36,6 @@ class JackMultiEngine:public AudioOut
         struct jack_multi *impl;
 };
 
+}
+
 #endif
