@@ -18,6 +18,8 @@
 
 #include "Engine.h"
 
+namespace zyn {
+
 /**This class is inherited by all the Midi input classes*/
 class MidiIn:public virtual Engine
 {
@@ -31,4 +33,6 @@ class MidiIn:public virtual Engine
                                 unsigned char value);
 };
 
+}
+
 #endif
