@@ -20,6 +20,8 @@
 #include "AudioOut.h"
 #include "MidiIn.h"
 
+namespace zyn {
+
 class NulEngine:public AudioOut, MidiIn
 {
     public:
@@ -44,4 +46,6 @@ class NulEngine:public AudioOut, MidiIn
         pthread_t     *pThread;
 };
 
+}
+
 #endif
