@@ -17,6 +17,8 @@
 #include "../globals.h"
 #include "AudioOut.h"
 
+namespace zyn {
+
 class OssMultiEngine : public AudioOut
 {
     public:
@@ -61,4 +63,6 @@ class OssMultiEngine : public AudioOut
         const char* linux_oss_wave_out_dev;
 };
 
+}
+
 #endif
