@@ -90,32 +90,44 @@ static const Ports voicePorts = {
         rDefault(Sound), "Type of Sound"),
     rParamZyn(PDelay,        rShort("delay"), rDefault(0),
         "Voice Startup Delay"),
-    rToggle(Presonance,      rShort("enable"), "Resonance Enable"),
-    rParamI(Pextoscil,       rShort("ext."),
+    rToggle(Presonance,      rShort("enable"), rDefault(true),
+        "Resonance Enable"),
+    rParamI(Pextoscil,       rDefault(-1),     rShort("ext."),
             rMap(min, -1), rMap(max, 16), "External Oscillator Selection"),
-    rParamI(PextFMoscil,     rShort("ext."),
+    rParamI(PextFMoscil,     rDefault(-1),     rShort("ext."),
             rMap(min, -1), rMap(max, 16), "External FM Oscillator Selection"),
-    rParamZyn(Poscilphase,   rShort("phase"),    "Oscillator Phase"),
-    rParamZyn(PFMoscilphase, rShort("phase"),  "FM Oscillator Phase"),
-    rToggle(Pfilterbypass,   rShort("bypass"), "Filter Bypass"),
+    rParamZyn(Poscilphase,   rShort("phase"),  rDefault(64),
+        "Oscillator Phase"),
+    rParamZyn(PFMoscilphase, rShort("phase"),  rDefault(64),
+        "FM Oscillator Phase"),
+    rToggle(Pfilterbypass,   rShort("bypass"), rDefault(false),
+        "Filter Bypass"),
 
     //Freq Stuff
-    rToggle(Pfixedfreq,           rShort("fixed"),  "If frequency is fixed"),
-    rParamZyn(PfixedfreqET,       rShort("e.t."),   "Equal Temperament Parameter"),
-    rParamZyn(PBendAdjust,        rShort("bend"),   "Pitch bend adjustment"),
-    rParamZyn(POffsetHz,          rShort("offset"), "Voice constant offset"),
+    rToggle(Pfixedfreq,      rShort("fixed"),  rDefault(false),
+        "If frequency is fixed"),
+    rParamZyn(PfixedfreqET,  rShort("e.t."),   rDefault(0),
+        "Equal Temperament Parameter"),
+    rParamZyn(PBendAdjust,   rShort("bend"),   rDefault(88) /* 64 + 24 */,
+        "Pitch bend adjustment"),
+    rParamZyn(POffsetHz,     rShort("offset"), rDefault(64),
+        "Voice constant offset"),
     //nominally -8192..8191
-    rParamI(PDetune,              rShort("fine"),
-            rLinear(0, 16383), "Fine Detune"),
-    rParamI(PCoarseDetune,        rShort("coarse"), "Coarse Detune"),
+    rParamI(PDetune,              rShort("fine"),       rLinear(0, 16383),
+        rDefault(8192),           "Fine Detune"),
+    rParamI(PCoarseDetune,        rShort("coarse"),     rDefault(0),
+        "Coarse Detune"),
     rParamZyn(PDetuneType,        rShort("type"),
-            rOptions(L35cents, L10cents, E100cents, E1200cents),
-            "Magnitude of Detune"),
-    rToggle(PFreqEnvelopeEnabled, rShort("enable"), "Frequency Envelope Enable"),
-    rToggle(PFreqLfoEnabled,      rShort("enable"), "Frequency LFO Enable"),
+        rOptions(L35cents, L10cents, E100cents, E1200cents), rDefault(L35cents),
+        "Magnitude of Detune"),
+    rToggle(PFreqEnvelopeEnabled, rShort("enable"),     rDefault(false),
+        "Frequency Envelope Enable"),
+    rToggle(PFreqLfoEnabled,      rShort("enable"),     rDefault(false),
+        "Frequency LFO Enable"),
 
     //Amplitude Stuff
-    rParamZyn(PPanning,                  rShort("pan."), "Panning"),
+    rParamZyn(PPanning,                  rShort("pan."), rDefault(64),
+        "Panning"),
     rParamZyn(PVolume,                   rShort("vol."), rDefault(100),
         "Volume"),
     rToggle(PVolumeminus,                rShort("inv."), rDefault(false),
@@ -370,7 +382,8 @@ static const Ports adPorts = {//XXX 16 should not be hard coded
     rArrayPaste,
     rRecurs(VoicePar, NUM_VOICES),
     {"VoicePar#" STRINGIFY(NUM_VOICES) "/Enabled::T:F",
-     rProp(parameter) rShort("enable") rDoc("Voice Enable"),
+     rProp(parameter) rShort("enable") rDoc("Voice Enable")
+     rDefault([true false false ...]),
      NULL, rArrayTCbMember(VoicePar, Enabled)},
     rRecur(GlobalPar, "Adnote Parameters"),
 };
@@ -400,17 +413,17 @@ ADnoteGlobalParam::ADnoteGlobalParam(const AbsTime *time_) :
         time(time_), last_update_timestamp(0)
 {
     FreqEnvelope = new EnvelopeParams(0, 0, time_);
-    FreqEnvelope->init(EnvelopeParams::ad_global_freq_env);
-    FreqLfo = new LFOParams(70, 0, 64, 0, 0, 0, 0, 0, time_);
+    FreqEnvelope->init(ad_global_freq);
+    FreqLfo = new LFOParams(ad_global_freq, time_);
 
     AmpEnvelope = new EnvelopeParams(64, 1, time_);
-    AmpEnvelope->init(EnvelopeParams::ad_global_amp_env);
-    AmpLfo = new LFOParams(80, 0, 64, 0, 0, 0, 0, 1, time_);
+    AmpEnvelope->init(ad_global_amp);
+    AmpLfo = new LFOParams(ad_global_amp, time_);
 
-    GlobalFilter   = new FilterParams(2, 94, 40, time_);
+    GlobalFilter   = new FilterParams(ad_global_filter, time_);
     FilterEnvelope = new EnvelopeParams(0, 1, time_);
-    FilterEnvelope->init(EnvelopeParams::ad_global_filter_env);
-    FilterLfo = new LFOParams(80, 0, 64, 0, 0, 0, 0, 2, time_);
+    FilterEnvelope->init(ad_global_filter);
+    FilterLfo = new LFOParams(ad_global_filter, time_);
     Reson     = new Resonance();
 }
 
@@ -556,22 +569,22 @@ void ADnoteVoiceParam::enable(const SYNTH_T &synth, FFTwrapper *fft,
     FMSmp    = new OscilGen(synth, fft, NULL);
 
     AmpEnvelope = new EnvelopeParams(64, 1, time);
-    AmpEnvelope->init(EnvelopeParams::ad_voice_amp_env);
-    AmpLfo = new LFOParams(90, 32, 64, 0, 0, 30, 0, 1, time);
+    AmpEnvelope->init(ad_voice_amp);
+    AmpLfo = new LFOParams(ad_voice_amp, time);
 
     FreqEnvelope = new EnvelopeParams(0, 0, time);
-    FreqEnvelope->init(EnvelopeParams::ad_voice_freq_env);
-    FreqLfo = new LFOParams(50, 40, 0, 0, 0, 0, 0, 0, time);
+    FreqEnvelope->init(ad_voice_freq);
+    FreqLfo = new LFOParams(ad_voice_freq, time);
 
-    VoiceFilter    = new FilterParams(2, 50, 60, time);
+    VoiceFilter    = new FilterParams(ad_voice_filter, time);
     FilterEnvelope = new EnvelopeParams(0, 0, time);
-    FilterEnvelope->init(EnvelopeParams::ad_voice_filter_env);
-    FilterLfo = new LFOParams(50, 20, 64, 0, 0, 0, 0, 2, time);
+    FilterEnvelope->init(ad_voice_filter);
+    FilterLfo = new LFOParams(ad_voice_filter, time);
 
     FMFreqEnvelope = new EnvelopeParams(0, 0, time);
-    FMFreqEnvelope->init(EnvelopeParams::ad_voice_fm_freq_env);
+    FMFreqEnvelope->init(ad_voice_fm_freq);
     FMAmpEnvelope = new EnvelopeParams(64, 1, time);
-    FMAmpEnvelope->init(EnvelopeParams::ad_voice_fm_amp_env);
+    FMAmpEnvelope->init(ad_voice_fm_amp);
 }
 
 /*
