@@ -36,8 +36,10 @@ constexpr int sizeof_pvowels = sizeof(FilterParams::Pvowels);
 
 static const rtosc::Ports subsubports = {
     rParamZyn(freq, rShort("f.freq"), "Formant frequency"),
-    rParamZyn(amp,  rShort("f.str"),  "Strength of formant"),
-    rParamZyn(q,    rShort("f.q"),    "The formant's quality factor, also known as resonance bandwidth or Q for short"),
+    rParamZyn(amp,  rShort("f.str"),  rDefault(127), "Strength of formant"),
+    rParamZyn(q,    rShort("f.q"),    rDefault(64),
+              "The formant's quality factor, also known as "
+              "resonance bandwidth or Q for short"),
 };
 #undef rObject
 
@@ -63,35 +65,56 @@ const rtosc::Ports FilterParams::ports = {
     rSelf(FilterParams),
     rPaste,
     rArrayPaste,
+    rOption(loc, rProp(internal),
+            rOptions(ad_global_filter, ad_voice_filter, sub_filter, in_effect),
+            "location of the filter"),
     rOption(Pcategory,          rShort("class"),
-            rOptions(analog, formant, st.var.), "Class of filter"),
+            rOptions(analog, formant, st.var.), rDefault(analog),
+            "Class of filter"),
     rOption(Ptype,              rShort("type"),
-            rOptions(LP1, HP1, LP2, HP2, BP, notch, peak,
-                l.shelf, h.shelf), "Filter Type"),
+            rOptions(LP1, HP1, LP2, HP2, BP, notch, peak, l.shelf, h.shelf),
+            rDefault(LP2), "Filter Type"),
     rParamI(Pstages,            rShort("stages"),
-            rLinear(0,5),  "Filter Stages"),
+            rLinear(0,5), rDefault(0), "Filter Stages"),
     rParamF(baseq,               rShort("q"),      rUnit(none),  rLog(0.1, 1000),
+            rDefaultDepends(loc),
+            rPreset(ad_global_filter, 0x1.1592acp+0),
+            rPreset(ad_voice_filter, 0x1.e2f3ap+1),
+            rPreset(sub_filter, 0x1.1592acp+0),
+            rPreset(in_effect, 0x1.384298p+2),
             "Quality Factor (resonance/bandwidth)"),
-    rParamF(basefreq,           rShort("cutoff"),  rUnit(Hz),    rLog(31.25, 32000),
+    rParamF(basefreq,           rShort("cutoff"),
+            rUnit(Hz),    rLog(31.25, 32000),
+            rDefaultDepends(loc),
+            rPreset(ad_global_filter, 0x1.3d434p+12),
+            rPreset(ad_voice_filter, 0x1.d48ab6p+8),
+            rPreset(sub_filter, 0x1.294d3ep+11),
+            rPreset(in_effect, 0x1.f3fffcp+9),
             "Base cutoff frequency"),
-    rParamF(freqtracking,       rShort("f.track"), rUnit(%),     rLinear(-100, 100),
+    rParamF(freqtracking,       rShort("f.track"), rUnit(%),
+            rLinear(-100, 100), rDefault(0.0f),
             "Frequency Tracking amount"),
-    rParamF(gain,               rShort("gain"),    rUnit(dB),    rLinear(-30, 30),
+    rParamF(gain,               rShort("gain"),    rUnit(dB),
+            rLinear(-30, 30),   rDefault(0.0f),
             "Output Gain"),
     rParamI(Pnumformants,       rShort("formants"),
-            rLinear(1,12),  "Number of formants to be used"),
+            rLinear(1,12),      rDefault(3),
+            "Number of formants to be used"),
     rParamZyn(Pformantslowness, rShort("slew"),
-            "Rate that formants change"),
-    rParamZyn(Pvowelclearness,  rShort("clarity"),
+            rDefault(64), "Rate that formants change"),
+    rParamZyn(Pvowelclearness,  rShort("clarity"), rDefault(64),
             "How much each vowel is smudged with the next in sequence. A high clarity will avoid smudging."),
-    rParamZyn(Pcenterfreq,      rShort("cutoff"),
+    rParamZyn(Pcenterfreq,      rShort("cutoff"),  rDefault(64),
             "Center Freq (formant)"),
-    rParamZyn(Poctavesfreq,     rShort("octaves"),
+    rParamZyn(Poctavesfreq,     rShort("octaves"), rDefault(64),
             "Number of octaves for formant"),
 
-    rParamI(Psequencesize,    rShort("seq.size"), rLinear(0, FF_MAX_SEQUENCE), "Length of vowel sequence"),
-    rParamZyn(Psequencestretch, rShort("seq.str"), "How modulators stretch the sequence"),
-    rToggle(Psequencereversed,  rShort("reverse"), "If the modulator input is inverted"),
+    rParamI(Psequencesize,    rShort("seq.size"),
+            rLinear(0, FF_MAX_SEQUENCE), rDefault(3), "Length of vowel sequence"),
+    rParamZyn(Psequencestretch, rShort("seq.str"),
+            rDefault(40), "How modulators stretch the sequence"),
+    rToggle(Psequencereversed,  rShort("reverse"),
+            rDefault(false), "If the modulator input is inverted"),
 
     {"vowel_seq#" STRINGIFY(FF_MAX_SEQUENCE) "::i", rShort("vowel") rDoc("Vowel number of this sequence position"), NULL,
         [](const char *msg, RtData &d){
@@ -116,7 +139,10 @@ const rtosc::Ports FilterParams::ports = {
             d.reply(d.loc, "b", sizeof_pvowels, obj->Pvowels);
         }},
 
-    {"Pvowels#" STRINGIFY(FF_MAX_VOWELS) "/", NULL, &subports,
+    rEnabledCondition(is_formant_filter, obj->Pcategory == 1),
+    {"Pvowels#" STRINGIFY(FF_MAX_VOWELS) "/",
+        rEnabledByCondition(is_formant_filter),
+        &subports,
         [](const char *msg, RtData &d) {
             const char *mm = msg; \
             while(*mm && !isdigit(*mm)) ++mm; \
@@ -172,7 +198,6 @@ const rtosc::Ports FilterParams::ports = {
                             0.0,     cf.d[1]);
                 }
             } else if(obj->Pcategory == 2) {
-                int order = 0;
                 float gain = dB2rap(obj->getgain());
                 auto cf = SVFilter::computeResponse(obj->Ptype,
                         Filter::getrealfreq(obj->getfreq()),
@@ -292,25 +317,54 @@ const rtosc::Ports FilterParams::ports = {
 
 
 
-FilterParams::FilterParams(const AbsTime *time_)
-    :FilterParams(0,64,64, time_)
-{
-}
-FilterParams::FilterParams(unsigned char Ptype_,
-                           unsigned char Pfreq_,
-                           unsigned char Pq_,
-                           const AbsTime *time_):
-        time(time_), last_update_timestamp(0)
+void FilterParams::setup()
 {
     setpresettype("Pfilter");
-    Dtype = Ptype_;
-    Dfreq = Pfreq_;
-    Dq    = Pq_;
 
     changed = false;
     defaults();
 }
 
+FilterParams::FilterParams(const AbsTime *time_)
+    :FilterParams(in_effect, time_)
+{
+}
+
+FilterParams::FilterParams(unsigned char Ptype_,
+                           unsigned char Pfreq_,
+                           unsigned char Pq_,
+                           consumer_location_t loc,
+                           const AbsTime *time_):
+        loc(loc), time(time_), last_update_timestamp(0),
+        Dtype(Ptype_), Dfreq(Pfreq_), Dq(Pq_)
+{
+    setup();
+}
+
+FilterParams::FilterParams(consumer_location_t loc,
+                           const AbsTime *time_):
+        loc(loc), time(time_), last_update_timestamp(0)
+{
+    auto init =
+        [&](unsigned char Ptype_, unsigned char Pfreq_, unsigned char Pq_)
+    {
+        Dtype = Ptype_;
+        Dfreq = Pfreq_;
+        Dq    = Pq_;
+    };
+
+    switch(loc)
+    {
+        case ad_global_filter:  init(2, 94, 40); break;
+        case ad_voice_filter:   init(2, 50, 60); break;
+        case sub_filter:        init(2, 80, 40); break;
+        case in_effect:         init(0, 64, 64); break;
+        default: throw std::logic_error("Invalid filter consumer location");
+    }
+
+    setup();
+}
+
 FilterParams::~FilterParams()
 {}
 
@@ -363,8 +417,8 @@ void FilterParams::defaults(int n)
 /*
  * Get the parameters from other FilterParams
  */
-
-void FilterParams::getfromFilterParams(FilterParams *pars)
+// WARNING! Function unused since 2004, see declaration in header
+void FilterParams::getfromFilterParams(const FilterParams *pars)
 {
     defaults();
 
@@ -570,7 +624,7 @@ void FilterParams::getfromXML(XMLwrapper& xml)
         basefreq     = xml.getparreal("basefreq",   1000);
         baseq        = xml.getparreal("baseq",      10);
         gain         = xml.getparreal("gain",       0);
-        freqtracking = xml.getparreal("freq_track", 0);
+        freqtracking = xml.getparreal("freq_tracking", 0);
     }
 
     //formant filter parameters
