@@ -33,20 +33,40 @@ namespace zyn {
 static const rtosc::Ports _ports = {
     rSelf(LFOParams),
     rPaste,
-    rParamF(Pfreq, rShort("freq"), rLinear(0.0,1.0), "frequency of LFO\n"
+    rOption(loc, rProp(internal),
+            rOptions(ad_global_amp, ad_global_freq, ad_global_filter,
+                     ad_voice_amp, ad_voice_freq, ad_voice_filter, unspecified),
+            "location of the filter"),
+    rParamF(Pfreq, rShort("freq"), rLinear(0.0,1.0),
+            rDefaultDepends(loc),
+            rPreset(ad_global_amp, 0x1.42850ap-1), // 80
+            rPreset(ad_global_freq, 0x1.1a3468p-1), // 70
+            rPreset(ad_global_filter, 0x1.42850ap-1),
+            rPreset(ad_voice_amp, 0x1.6ad5acp-1), // 90
+            rPreset(ad_voice_freq, 0x1.93264cp-2), // 50
+            rPreset(ad_voice_filter, 0x1.93264cp-2),
+            "frequency of LFO\n"
             "lfo frequency = (2^(10*Pfreq)-1)/12 * stretch\n"
             "true frequency is [0,85.33] Hz"),
-    rParamZyn(Pintensity, rShort("depth"), "Intensity of LFO"),
-    rParamZyn(Pstartphase, rShort("start"), rSpecial(random), "Starting Phase"),
+    rParamZyn(Pintensity, rShort("depth"),
+              rDefaultDepends(loc),
+              rDefault(0), rPreset(ad_voice_amp, 32),
+              rPreset(ad_voice_freq, 40), rPreset(ad_voice_filter, 20),
+              "Intensity of LFO"),
+    rParamZyn(Pstartphase, rShort("start"), rSpecial(random),
+              rDefaultDepends(loc), rDefault(64), rPreset(ad_voice_freq, 0),
+              "Starting Phase"),
     rOption(PLFOtype, rShort("type"), rOptions(sine, triangle, square, up, down,
-                exp1, exp2), "Shape of LFO"),
-    rParamZyn(Prandomness, rShort("a.r."), rSpecial(disable),
+                exp1, exp2), rDefault(sine), "Shape of LFO"),
+    rParamZyn(Prandomness, rShort("a.r."), rSpecial(disable), rDefault(0),
             "Amplitude Randomness (calculated uniformly at each cycle)"),
     rParamZyn(Pfreqrand, rShort("f.r."), rSpecial(disable), rDefault(0),
             "Frequency Randomness (calculated uniformly at each cycle)"),
-    rParamZyn(Pdelay, rShort("delay"), rSpecial(disable), "Delay before LFO start\n"
-            "0..4 second delay"),
-    rToggle(Pcontinous, rShort("c"), "Enable for global operation"),
+    rParamZyn(Pdelay, rShort("delay"), rSpecial(disable),
+              rDefaultDepends(loc), rDefault(0), rPreset(ad_voice_amp, 30),
+              "Delay before LFO start\n0..4 second delay"),
+    rToggle(Pcontinous, rShort("c"), rDefault(false),
+            "Enable for global operation"),
     rParamZyn(Pstretch, rShort("str"), rCentered, rDefault(64),
         "Note frequency stretch"),
 // these are currently not yet implemented at must be hidden therefore
@@ -69,20 +89,40 @@ static const rtosc::Ports _ports = {
 
 const rtosc::Ports &LFOParams::ports = _ports;
 
-LFOParams::LFOParams(const AbsTime *time_) : time(time_)
+void LFOParams::setup()
 {
-    Dfreq       = 64;
-    Dintensity  = 0;
-    Dstartphase = 0;
-    DLFOtype    = 0;
-    Drandomness = 0;
-    Ddelay      = 0;
-    Dcontinous  = 0;
-    fel  = 0;
+    switch(loc) {
+        case loc_unspecified:
+            fel = consumer_location_type_t::unspecified;
+            break;
+        case ad_global_freq:
+        case ad_voice_freq:
+            fel = consumer_location_type_t::freq;
+            setpresettype("Plfofrequency");
+            break;
+        case ad_global_amp:
+        case ad_voice_amp:
+            fel = consumer_location_type_t::amp;
+            setpresettype("Plfoamplitude");
+            break;
+        case ad_global_filter:
+        case ad_voice_filter:
+            fel = consumer_location_type_t::filter;
+            setpresettype("Plfofilter");
+            break;
+        default:
+            throw std::logic_error("Invalid envelope consumer location");
+    }
 
     defaults();
 }
 
+// TODO: reuse
+LFOParams::LFOParams(const AbsTime *time_) :
+    LFOParams(64, 0, 0, 0, 0, 0, 0, loc_unspecified, time_)
+{
+}
+
 LFOParams::LFOParams(char Pfreq_,
                      char Pintensity_,
                      char Pstartphase_,
@@ -90,20 +130,10 @@ LFOParams::LFOParams(char Pfreq_,
                      char Prandomness_,
                      char Pdelay_,
                      char Pcontinous_,
-                     char fel_,
-                     const AbsTime *time_) : time(time_),
+                     consumer_location_t loc,
+                     const AbsTime *time_) : loc(loc),
+                                             time(time_),
                                              last_update_timestamp(0) {
-    switch(fel_) {
-        case 0:
-            setpresettype("Plfofrequency");
-            break;
-        case 1:
-            setpresettype("Plfoamplitude");
-            break;
-        case 2:
-            setpresettype("Plfofilter");
-            break;
-    }
     Dfreq       = Pfreq_;
     Dintensity  = Pintensity_;
     Dstartphase = Pstartphase_;
@@ -111,9 +141,40 @@ LFOParams::LFOParams(char Pfreq_,
     Drandomness = Prandomness_;
     Ddelay      = Pdelay_;
     Dcontinous  = Pcontinous_;
-    fel  = fel_;
 
-    defaults();
+    setup();
+}
+
+LFOParams::LFOParams(consumer_location_t loc,
+                     const AbsTime *time_) : loc(loc),
+                                             time(time_),
+                                             last_update_timestamp(0) {
+
+    auto init =
+        [&](char Pfreq_, char Pintensity_, char Pstartphase_, char PLFOtype_,
+            char Prandomness_, char Pdelay_, char Pcontinous_)
+    {
+        Dfreq       = Pfreq_;
+        Dintensity  = Pintensity_;
+        Dstartphase = Pstartphase_;
+        DLFOtype    = PLFOtype_;
+        Drandomness = Prandomness_;
+        Ddelay      = Pdelay_;
+        Dcontinous  = Pcontinous_;
+    };
+
+    switch(loc)
+    {
+        case ad_global_amp:    init(80, 0, 64, 0, 0, 0, 0); break;
+        case ad_global_freq:   init(70, 0, 64, 0, 0, 0, 0); break;
+        case ad_global_filter: init(80, 0, 64, 0, 0, 0, 0); break;
+        case ad_voice_amp:     init(90, 32, 64, 0, 0, 30, 0); break;
+        case ad_voice_freq:    init(50, 40,  0, 0, 0,  0, 0); break;
+        case ad_voice_filter:  init(50, 20, 64, 0, 0,  0, 0); break;
+        default: throw std::logic_error("Invalid LFO consumer location");
+    }
+
+    setup();
 }
 
 LFOParams::~LFOParams()
