@@ -18,6 +18,8 @@
 #include "../globals.h"
 #include "Presets.h"
 
+namespace zyn {
+
 class SUBnoteParameters:public Presets
 {
     public:
@@ -112,4 +114,6 @@ class SUBnoteParameters:public Presets
         static const rtosc::Ports &ports;
 };
 
+}
+
 #endif
