@@ -18,6 +18,8 @@
 #include "../Misc/Time.h"
 #include "WatchPoint.h"
 
+namespace zyn {
+
 /**Class for creating Low Frequency Oscillators*/
 class LFO
 {
@@ -70,4 +72,6 @@ class LFO
         void computeNextFreqRnd(void);
 };
 
+}
+
 #endif
