@@ -92,10 +92,14 @@ const rtosc::Ports OscilGen::non_realtime_ports = {
               "modulation parameter"),
     rParamZyn(Pmodulationpar3, rShort("p3"), rDefault(32),
               "modulation parameter"),
+    rToggle(ADvsPAD, rShort("If it is used by PADSynth"),
+            "If it is used by PADSynth (and not ADSynth)"),
 
 
     //TODO update to rArray and test
-    {"phase#128::c:i", rProp(parameter) rLinear(0,127) rDoc("Sets harmonic phase"),
+    {"phase#128::c:i", rProp(parameter) rLinear(0,127)
+        rDefault([64 ...])
+        rDoc("Sets harmonic phase"),
         NULL, [](const char *m, rtosc::RtData &d) {
             const char *mm = m;
             while(*mm && !isdigit(*mm)) ++mm;
@@ -118,7 +122,8 @@ const rtosc::Ports OscilGen::non_realtime_ports = {
             }
         }},
     //TODO update to rArray and test
-    {"magnitude#128::c:i", rProp(parameter) rLinear(0,127) rDoc("Sets harmonic magnitude"),
+    {"magnitude#128::c:i", rProp(parameter) rLinear(0,127)
+        rDefault([127 64 64 ...]) rDoc("Sets harmonic magnitude"),
         NULL, [](const char *m, rtosc::RtData &d) {
             //printf("I'm at '%s'\n", d.loc);
             const char *mm = m;
@@ -198,7 +203,9 @@ const rtosc::Ports OscilGen::non_realtime_ports = {
 const rtosc::Ports OscilGen::realtime_ports{
     rSelf(OscilGen),
     rPresetType,
-    rParamZyn(Prand, rLinear(-64, 63), rShort("phase rnd"), "Oscillator Phase Randomness: smaller than 0 is \""
+    rParamZyn(Prand, rLinear(-64, 63), rShort("phase rnd"),
+            rDefaultDepends(ADvsPAD), rPreset(true, 127), rPreset(false, 64),
+            "Oscillator Phase Randomness: smaller than 0 is \""
             "group\", larger than 0 is for each harmonic"),
     rParamZyn(Pamprandpower, rShort("variance"), rDefault(64),
             "Variance of harmonic randomness"),
