@@ -17,6 +17,8 @@
 #include "SynthNote.h"
 #include "../globals.h"
 
+namespace zyn {
+
 class SUBnote:public SynthNote
 {
     public:
@@ -104,4 +106,6 @@ class SUBnote:public SynthNote
         WatchManager *wm;
 };
 
+}
+
 #endif
