@@ -14,6 +14,8 @@
 #define SYNTH_NOTE_H
 #include "../globals.h"
 
+namespace zyn {
+
 class Allocator;
 class Controller;
 struct SynthParams
@@ -111,4 +113,6 @@ class SynthNote
         WatchManager     *wm;
 };
 
+}
+
 #endif
