@@ -25,13 +25,11 @@
 #include <err.h>
 #endif
 #include <unistd.h>
-#include <pthread.h>
 
 #include <getopt.h>
 
 #include <rtosc/rtosc.h>
 #include <rtosc/ports.h>
-#include <rtosc/thread-link.h>
 #include "Params/PADnoteParameters.h"
 
 #include "DSP/FFTwrapper.h"
@@ -228,7 +226,6 @@ int main(int argc, char *argv[])
     cerr
     << "                Copyright (c) 2009-2017 Mark McCurry [active maintainer]"
     << endl;
-    cerr << "Compiled: " << __DATE__ << " " << __TIME__ << endl;
     cerr << "This program is free software (GNU GPL v2 or later) and \n";
     cerr << "it comes with ABSOLUTELY NO WARRANTY.\n" << endl;
     if(argc == 1)
@@ -329,7 +326,7 @@ int wmidi = -1;
         /**\todo check this process for a small memory leak*/
         opt = getopt_long(argc,
                           argv,
-                          "l:L:M:r:b:o:I:O:N:e:P:A:D:hvapSDUYZ",
+                          "l:L:M:r:b:o:I:O:N:e:P:A:d:D:hvapSDUYZ",
                           opts,
                           &option_index);
         char *optarguments = optarg;
